#include "assert_failed.hh"

#ifndef NDEBUG

#include "e_mail.hh"
#include "string.hh"
#include "io.hh"
#include "main.hh"
//#include "text.hh"
#include "write_formatted.hh"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h> 

void (*assert_failed_hooks)()= NULL;
/* The hook function. 
 */

void
Assert_Failed(const char *file, 
	      int         line,
	      const char *text,
	      const char *function)
{
	/* Avoid recursive failing of assertions. 
	 */
	static bool failed= false;

	if (! failed)
	{
		failed= true;

		String name= Main_Get_Name();
		Write_Formatted(IO::err,
			    "\n *** internal error in Baby process *** \n"
			    "error:  assertion failed in %s:%d:\n"
			    "        in function %s\n"
			    "        %s\n"
			    "PID= %d, prog= %s\n"
			    "Sleeping.  You can now attach a debugger to me.\n"
			    "Send any signal to me to continue.\n",
			    file,
			    line,
			    function,
			    text,
			    (int)getpid(),
				name());
	}
	else
		fprintf(stderr,
			"*** assertion failed inside Assert_Failed().\n");

	/* Call hook */
	if (assert_failed_hooks != NULL)
		assert_failed_hooks();

	pause();
}

void
Set_Assert_Hook(void (*f)())
{
	if (assert_failed_hooks == NULL)
		assert_failed_hooks= f;
}

#endif /* ! NDEBUG */

