#ifndef BUTTON_BAR_INCLUDED
#define BUTTON_BAR_INCLUDED

#include "packer.hh"
#include "text_button.hh"
#include "trl.hh"
#include "vector.hh"
#include "buffer.hh"
#include "string.hh"

template <class T>
class Button_Bar
	:  public T
{
public:

	bool Open(Pointer <Widget>);

	void Append(int msg,
		 String text,
		 bool enabled= true);
	void Append_Cancel_Button();
	void Append_OK_Button();
	void Append_Apply_Button();
	void Append_Reset_Button(); 
	
	Text_Button &Get(int index)
		{
			return buttons[index](); 
		}

private:

	Vector <Dynamic <Text_Button> > buttons;

	template <class U> 
	bool Append(typename U::Open_Info open_info)
		{
			if (! buttons.Set(*buttons + 1))
				return false;
			Dynamic <U> u;
			if (! u.Alloc())
			{
				buttons= -1;
				return false;
			}
			if (! u().Open(this,
				       open_info))
			{
				buttons= -1;
				return false;
			}
			u.Transfer_To(buttons[-1]);
			return true; 
		}
};

template <class T>
bool Button_Bar <T> ::Open(Pointer <Widget> parent)
{
	if (T::Open(parent /* , unit / 3 */ ))
	{
		return true;
	}
	else
		return false;
}

template <class T>
void Button_Bar <T> ::Append(int msg_click,
			  String text,
			  bool enabled)
{
	Append <Text_Button> (Text_Button::Open_Info(msg_click, text, enabled));
}

template <class T>
void Button_Bar <T> ::Append_Cancel_Button()
{
	Append <Text_Button> (Text_Button::Open_Info(msg_cancel,
						  Trl("Cancel")));
}

template <class T>
void Button_Bar <T> ::Append_Reset_Button()
{
	Append <Text_Button> (Text_Button::Open_Info(msg_reset,
						  Trl("Reset")));
}

template <class T>
void Button_Bar <T> ::Append_OK_Button()
{
	Append <Text_Button> (Text_Button::Open_Info(msg_ok,
						     Trl("OK")));
}

template <class T>
void Button_Bar <T> ::Append_Apply_Button()
{
	Append <Text_Button> (Text_Button::Open_Info(msg_apply,
						  Trl("Apply")));
}

typedef Button_Bar <V_Packer> V_Button_Bar;
typedef Button_Bar <H_Packer> H_Button_Bar;

#endif /* ! BUTTON_BAR_INCLUDED */
