#include "canvas.hh"

#include "client_image.hh"
#include "dbg.hh"

#include <limits.h> /* ULONG_MAX */

GdkFont *Canvas::font= NULL;

Canvas::Color::Color(RGB rgb)
{
	Dbg_("Allocating:  %d, %d, %d",
	    rgb.r, rgb.g, rgb.b);

	red=   rgb.r * 0x100;
	green= rgb.g * 0x100;
	blue=  rgb.b * 0x100;

	if (FALSE == gdk_colormap_alloc_color
	    (gdk_colormap_get_system(),
	     this,
	     FALSE,
	     TRUE))
	{
		Dbg_("\tFailed.  Using white.");
		gdk_color_white(gdk_colormap_get_system(),
				this);
	}
	else
		Dbg_("\tSuccess.");
}

void
Canvas::Draw_Ellipse(Rect rect)
{
	Assert (Is()); 

	gdk_draw_arc(drawable,
		     gc,
		     TRUE,
		     rect.origin.x,
		     rect.origin.y,
		     rect.size.x,
		     rect.size.y,
		     0,
		     64 * 360);
}

void
Canvas::Draw_Canvas(const Canvas &canvas,
		    Rect dst,
		    Point src)
{
	Assert (Is() && canvas.Is());

	gdk_draw_pixmap(drawable,
			gc,
			canvas.drawable,
			src.x, src.y,
			dst.origin.x, dst.origin.y,
			dst.size.x, dst.size.y);
}

void Canvas::
Draw_Stretch_Canvas(const Canvas &canvas,
		    Rect dst,
		    Rect src,
		    bool anti_alias)
{
	Client_Image::Draw_Stretch_Canvas(This, canvas, dst, src, anti_alias); 
}

void
Canvas::Draw_Label(Point point,
		   String text, 
		   Display_Type /* display_type */ )
{
	Dbg_Function();

	Assert (Is());

	if (! Load_Font())
		return;

	gdk_draw_text(drawable,
		      font,
		      gc,
		      point.x, point.y,
//		      rect.origin.x,
//		      rect.origin.y  + rect.size.y  ,
		      text(),
		      *text);
}

void Canvas::Draw_Line(Point from,
			   Point to)
{
  Assert (Is());

  gdk_draw_line(drawable, gc,
		from.x, from.y,
		to.x, to.y);
}


void Canvas::Set_Draw_Mode(Draw_Mode draw_mode)
		{
			Assert (Is());
#if 0
			if (true)
			  {
			    if (draw_mode == dm_or)
			      draw_mode= dm_and;
			    else if (draw_mode == dm_and)
			      draw_mode= dm_or; 
			  }
#endif /* 0 */ 
			gdk_gc_set_function(gc,
					    (GdkFunction)draw_mode);
		}

bool Canvas::Load_Font()
{
	if (font != NULL)
		return true; 

	const char *font_name=
		"-*-helvetica-bold-r-normal--*-120-*-*-*-*-iso8859-1"; 

	font= gdk_font_load(font_name);

	if (font == NULL)
	{
		return false;
	}

	return true; 
}

int Canvas::Get_Label_Width(String text)
{
	Assert (text.Is());

	if (! Load_Font())
		return 1;

	gint lbearing, rbearing, width, ascent, descent;

	gdk_text_extents(font,
			 text(),
			 *text,
			 & lbearing,
			 & rbearing,
			 & width,
			 & ascent,
			 & descent);

	return width; 
}

