#include "child_process.hh"

#include "file_name.hh"
#include "vector.hh"
//#include "token.hh"

Child_Process::Child_Process()
	:  child(PROCESS_NO)
{
}

Child_Process::~Child_Process()
{
	if (Is())
		Close();
}

bool
Child_Process::Is() const
{
	return child != PROCESS_NO;
}

bool
Child_Process::Open(String program_file_name,
		    const Vector <String> &args,
		    Stream *in,
		    Stream *out,
		    Stream *err,
		    bool nice)
{
	Assert (! Is());

	if (! Process_Create(program_file_name,
			     args,
			     false,
			     & child,
			     in,
			     out,
			     err,
			     nice))
		return false;

	paused= false;

	return true;
}

void
Child_Process::Close()
{
	Assert (Is());
	    
	Process_Kill(child);
	child= PROCESS_NO;
}

int
Child_Process::Wait()
{
	Assert (Is());

	int exit_code;
	if (! Process_Wait(child, exit_code))
		exit_code= 1;

	child= PROCESS_NO;
	    
	return exit_code;
}

void
Child_Process::Pause()
{
	Assert (Is());

	if (paused)
	{
		Process_Resume(child);
	}
	else
	{
		Process_Pause(child);
	}

	paused= ! paused;
}
