#ifndef CLIENT_IMAGE_INCLUDED
#define CLIENT_IMAGE_INCLUDED

#include "point.hh"
#include "pointer.hh"
#include "canvas.hh"

class Client_Image
	:  public Pointed
{
public:

	typedef guint32 Pixel; 

	Client_Image()
		:  image(NULL)
		{
		}
	bool Is() const
		{
			return image != NULL; 
		}
	void Free()
		{
			if (! Is())
				return;
			gdk_image_destroy(image);
			image= NULL; 
		}
	~Client_Image()
		{
			if (Is())
				Free(); 
		}

	Point Get_Size() const
		{
			Assert (Is());
			return Point(Deref(image).width,
				     Deref(image).height); 
		}
	void Get_Pixel(int x, 
		       int y,
		       int &r,
		       int &g,
		       int &b) const
		{
			guint32 pixel= gdk_image_get_pixel(image,
							   x, y);
			r= pixel & 0xff;
			g= (pixel >> 8) & 0xff;
			b= (pixel >> 8) & 0xff;
		}

	bool Create(Point size)
		{
			Assert (size.x > 0 && size.y > 0); 
			Assert (! Is()); 

			image= gdk_image_new(GDK_IMAGE_FASTEST,
					     gdk_rgb_get_visual(),
					     size.x,
					     size.y);

			return image != NULL; 
		}
	void Draw_Pixel(int x,
			int y,
			int r,
			int g,
			int b)
		{
			Assert (Is());

			guint32 pixel= 0;
			pixel |= r;
			pixel |= g << 8;
			pixel |= b << 16;

			gdk_image_put_pixel(image, x, y, pixel); 
		}

	bool Send_Rect(Canvas &, 
		       Rect ,
		       Point ); 
	bool Receive_Rect(const Canvas &,
			  Rect ); 

	static bool Draw_Stretch_Canvas(Canvas &,
					const Canvas &,
					Rect &,
					Rect &,
					bool anti_alias= true);

private:

	GdkImage *image; 
}; 

#endif /* ! CLIENT_IMAGE_INCLUDED */
