#include "document.hh"

#include "choice.hh"
#include "dbg.hh"

void
Document::Load()
{
	if (modified)
	{
		asked_for_what= 1;
//		asked_for_close= false;
		Document_Ask();
	}
	else
	{
		if (Document_Use_File_Name())
			Document_Ask_Open_Name();
		else
		{
			Document_Save(""); 
		}
	}
}

bool
Document::Save()
{
	if (Has_Name())
		return Do_Save();
	else
	{
		Save_As();
		return false; 
	}
}

void
Document::Save_As()
{
	Dbg_Function();
	asked_save_for_close= 1;
	Document_Ask_Save_Name();
}

bool
Document::Do_Save()
{
	Assert (file_name.Is());
	if (Document_Save(file_name))
	{
		modified= false;
		return true;
	}
	else
	{
		if (Document_Use_File_Name())
			Document_Save_Error(); 
		return false; 
	}
}

bool
Document::Asked_Open_Name(String new_file_name)
{
	Assert (new_file_name.Is());

	return Do_Load(new_file_name);
}

bool
Document::Do_Load(String new_file_name)
{
	Assert (new_file_name.Is());

	switch (Document_Load(new_file_name))
	{
	default:
		Assert_Unreached();
		return false;

	case lr_error:
		Document_Load_Error(new_file_name); 
		return false;

	case lr_success:
		if (Document_Use_File_Name())
		{
			Document_File_Name_Changed(file_name);
			file_name= new_file_name;
		}
		/* fall through */
	case lr_wrong_type:
		return true;
	}
}

void
Document::Asked_Save_Name(String new_file_name)
{
	Dbg_Function();

	Assert (new_file_name.Is());

	file_name= new_file_name;
	bool success= Do_Save(); 
	Dbg("\tDo_Save()= %d", success);

	Dbg("\tDocument_Use_File_Name()= %d",
	    Document_Use_File_Name()); 

	if (Document_Use_File_Name())
		Document_File_Name_Changed(file_name);

	if (! success)
		return;

	Dbg_Int(asked_save_for_close);

	switch (asked_save_for_close)
	{
	default:
		Assert_Unreached();
		break;

	case 0:
		Assert (Document_Use_File_Name()); 
		Document_Close();
		break;

	case 1:
		/* Do nothing */
		break;

	case 2:
		Document_Ask_Open_Name(); 
		break;

	case 3:
		modified= false;
		DO_New();
		break;
	}
}

void
Document::Open(String new_file_name)
{
	Assert (Document_Use_File_Name()); 
	file_name= new_file_name;
	Document_File_Name_Changed(file_name);
}

void
Document::Asked(int answer)
{
	switch (asked_for_what)
	{
	default:  Assert_Unreached();  break; 

	case 0: /* close */

		switch (answer)
		{
		default:
			Assert_Unreached();
			break;
			
		case ch_yes:
			if (Document_Use_File_Name())
			{
				if (file_name.Is())
				{
					if (Document_Save(file_name))
					{
						Document_Close();
					}
					else
					{
						Document_Save_Error(); 
					}
				}
				else
				{
					asked_save_for_close= 0; 
					Document_Ask_Save_Name(); 
				}
			}
			else
				Document_Save_Close(); 
			break;

		case ch_no:
			Document_Close();
			break;

		case ch_cancel:
			/* Do nothing */ 
			break;
		}
		break;

	case 1: /* load */
	case 2: /* new */
		switch (answer)
		{
		default:
			Assert_Unreached();
			break;

		case ch_yes:
			asked_save_for_close= asked_for_what == 1 ? 2 : 3;
			Document_Ask_Save_Name(); 
			break;

		case ch_no:
			if (asked_for_what == 1)
				Document_Ask_Open_Name(); 
			else
			{
				modified= false;
				DO_New();
			}
			break;

		case ch_cancel:
			/* Do nothing */
			break;
		}
		break;
	}
}

Document:: ~Document() 
{
}

bool Document::Can_Close()
{
	if (modified)
	{
		asked_for_what= 0;
//		asked_for_close= true; 
		Document_Ask();
		return false; 
	}
	else
		return true; 
}

void Document::New()
{
	if (! modified)
	{
		DO_New();
	}
	else
	{
		asked_for_what= 2;
		Document_Ask();
	}
}
