#ifndef DOCUMENT_INCLUDED
#define DOCUMENT_INCLUDED

#include "string.hh"

class Document
{
public:

	/* Return value of Document_Load(). 
	 */
	enum Load_Result
	{
		lr_error= 0, 
		lr_success= 1,
		lr_wrong_type= 2,
		/* File was loaded, but is of wrong type, thus is
		   cannot be overwritten. 
		 */
	};

	Document()
		:  modified (false) 
		{
		}
	virtual ~Document();
	
	void Open(String file_name);

	void Load();
	bool Save();
	/* Result:  Whether saving was done. 
	 */
	void Save_As();
	bool Asked_Open_Name(String file_name);
	void Asked_Save_Name(String file_name);
	void Asked(int answer);
	/* Return a constant from CHOICE:  ch_yes, ch_no or
	   ch_cancel. 
	 */
	bool Do_Load(String file_name);
	/* Load a specific file.  Return FALSE on error. 
	 */
	void Modify()
		{
			modified= true; 
		}
	void Unmodify()
		{
			modified= false; 
		}
	bool Can_Close();
	void New();

	bool Has_Name() const
		{
			return file_name.Is();
		}
	String Get_File_Name() const
		{
			return file_name; 
		}
	bool Get_Modified() const
		{
			return modified; 
		}

private:
	
	String file_name;
	/* If empty, no file name specified.
	 */
	bool modified;
	int asked_for_what;
//	bool asked_for_close;
	/* What was the reason for asking.
	 *    0:   closing
	 *    1:   loading another file name
	 *    2:   new
	 */
	int asked_save_for_close;
	/* What was the reason for asking the save file name.
	   0:  Save before closing.
	   1:  Save normally. 
	   2:  Save before loading. 
	   3:  Save before new. 
	 */

	bool Do_Save();
	/* Save to file.
	 */

	virtual Load_Result Document_Load(String file_name)= 0;
	virtual bool Document_Save(String file_name)= 0;
	virtual void Document_Ask_Open_Name()= 0;
	virtual void Document_Ask_Save_Name()= 0;
	virtual void Document_File_Name_Changed(String)= 0;
	virtual void Document_Ask()
		/* Ask "Object was changed...save?".  Call Asked()
		   later. 
		 */
		{
		}
	virtual void Document_Load_Error(String /* file_name */ )
		{
		}
	virtual void Document_Save_Error()
		{
		}
	virtual void Document_Close()
		/* Close document. 
		 */
		{
		}
	virtual bool Document_Use_File_Name() const= 0;
	virtual void Document_Save_Close()= 0;
	virtual void DO_New()= 0;
};

#endif /* ! DOCUMENT_INCLUDED */
