#ifndef GAME_ARCHIVE_INCLUDED
#define GAME_ARCHIVE_INCLUDED

/* Archive of chess games. 
 */

#include "vector.hh"
#include "game.hh"
#include "pointer.hh"
#include "raw_pgn.hh"
//#include "pgn_date.hh"

class Game_Archive
	:  public Pointed
{
public:

	bool Set_Size(int n);

	void Clear()
		{
			games.Free();
			raw.Free();
		}

	int operator * () const
		{
			return games.Get_Size();
		}

	Game &operator [] (int n) 
		{
			Assert (games[n].Is());
			return games[n]();
		}

	const Game &operator [] (int n) const
		{
			Assert (games[n].Is());
			return games[n]();
		}

	Raw_PGN &Get_Raw(int n)
		{
			return raw[n];
		}

	const Raw_PGN &Get_Raw(int n) const
		{
			return raw[n];
		}

	void Invalidate_Game(int n)
		{
			games[n].Free();
		}

	String Get_White(int i) const
		{
			if (games[i].Is())
				return games[i]().Get_Game_Info().white;
			String result= raw[i].Get_Tag("White");
			if (result == "?")
				result= "";
			return result;
		}
	String Get_Black(int i) const
		{
			if (games[i].Is())
				return games[i]().Get_Game_Info().black;
			String result= raw[i].Get_Tag("Black");
			if (result == "?") result= "";
			return result;
		}
	String Get_Event(int i) const
		{
			if (games[i].Is())
				return games[i]().Get_Game_Info().event;
			String result= raw[i].Get_Tag("Event");
			if (result == "?")  result= "";
			return result;
		}
	String Get_Round(int i) const
		{
			if (games[i].Is())
				return games[i]().Get_Game_Info().round;
			String result= raw[i].Get_Tag("Round");
			if (result == "?")  result= "";
			return result;
		}

	String Get_Date(int i) const
		{
			if (games[i].Is())
				return games[i]().Get_Game_Info().date;
			String result= raw[i].Get_Tag("Date");
			if (result == "?")  result= "";
			return result;
		}
	String Get_Site(int i) const
		{
			if (games[i].Is())
				return games[i]().Get_Game_Info().site;
			String result= raw[i].Get_Tag("Site");
			if (result == "?")  result= "";
			return result;
		}

	bool Check(int n);
	/* Return TRUE if game N exists. 
	 */
	bool Insert_Game(int index);
	void Remove(int n);
	void Set(int n,
		 const Game &game);
	bool Have_Game(int n) const
		{
			Assert (n >= 0 && n < *raw);
			return games[n].Is();
		}
	String Get_Result(int index);

private:

	Vector <Raw_PGN> raw;
	Vector <Dynamic <Game> > games;
	/* RAW and GAMES have the same size.  If GAMES[n] == NULL,
	   then RAW[n] contains the game, else GAMES[n]. 
	*/
};

#endif /* ! GAME_ARCHIVE_INCLUDED */
