#include "game_archive_editor.hh"

#include "keys.hh"

bool
Game_Archive_Editor::Document_Editor_Open(Pointer <Packer> parent)
{
	Vector <String> titles;
	if (! titles.Set(7))
		return false;
	titles[0]= Trl("Result");
	titles[1]= Trl("White");
	titles[2]= Trl("Black"); 
	titles[3]= Trl("Event");
	titles[4]= Trl("Round");
	titles[5]= Trl("Date");
	titles[6]= Trl("Site");

	parent().Set_Next_Resizeable(true);
	if (! game_list.Open(parent(),
			     Multi_List_Box::Open_Info
			     (7,
			      scr_both,
			      msg_list_clicked,
			      titles)))
		return false; 

	if (! button_bar.Open(parent()))
		return false; 

	button_bar.Append(msg_insert,
		       Trl("Insert"));
	button_bar.Append(msg_remove,
		       Trl("Remove"),
		       false);
	button_bar.Append(msg_edit,
		       Trl("Edit..."),
		       false);
	if (! status_bar.Open(parent(), 1))
		return false;
			
	Update_List();
	Update_Count();

	if (*game_archive == 1)
		/* If the archive contains exactly one game, open it
		   immediately. 
		*/
	{
		Edit(0); 
	}
	
	return true; 
}

void
Game_Archive_Editor::Editor_Window_Set_Value(Pointer <const Game_Archive> ga)
{
	if (ga != NULL)
		game_archive= ga();
}

const Game_Archive &
Game_Archive_Editor::Editor_Window_Get_Value()
{
	return game_archive;
}

void
Game_Archive_Editor::Menued_Get(int menu, Menued::Entries &entries)
{
	switch (menu)
	{
	default:
		Document_Editor <Game_Archive> ::Menued_Get(menu, entries);
		break;

	case menu_main_ext:
		entries.Append_Submenu(menu_edit,
				    Trl("Edit"));
		break;

	case menu_edit:
		entries.Append_Command(msg_insert, Trl("Insert game"));
		bool sel= game_list.Get_Selected() >= 0;
		entries.Append_Command(msg_remove, 
				       Trl("Remove game"),
				       key_no,
				       gl_nothing,
				       sel);
		entries.Append_Command(msg_edit, 
				       Trl("Edit game..."),
				       key_no,
				       gl_nothing,
				       sel);
				    
		break;
	}
}

bool
Game_Archive_Editor::Editor_Window_Change_Value(const Game_Archive &new_value)
{
	Close_Editors();
	game_archive= new_value;
	Update_List();
	Update_Count();
	return true;
}

void
Game_Archive_Editor::Update_List()
{
	if (! game_list.Is())
		return;
	
	game_list.Set_Row_Count(*game_archive); 

	for (int i= 0;  i < *game_archive;  ++i)
	{
		Update_Game_Info(i);
#if 0
		game_list.Set(0, i, game_archive.Get_Result(i));
		game_list.Set(1, i, game_archive.Get_Tag(i, "White"));
		game_list.Set(2, i, game_archive.Get_Tag(i, "Black"));
		game_list.Set(3, i, game_archive.Get_Tag(i, "Event"));
		game_list.Set(4, i, game_archive.Get_Tag(i, "Round"));
		game_list.Set(5, i, game_archive.Get_Tag(i, "Date"));
		game_list.Set(6, i, game_archive.Get_Tag(i, "Site"));
#endif /* 0 */ 
	}
}

void
Game_Archive_Editor::Update_Count()
{
	if (! status_bar.Is())
		return;

	if (! status_bar.Is(0))
		return;

	String text;
	if (*game_archive == 0)
		text= Trl("Archive is empty");
	else if (*game_archive == 1)
		text= Trl("1 game");
	else
		text= Format("%d games",
			     *game_archive);

	status_bar.Set(0, text);
}

void
Game_Archive_Editor::MR_Receive(int id)
{
	Dbg_Function(); 

	switch (id)
	{
	default: /* edited */ 
	{
		if (id < msg_edited)
		{
			Document_Editor <Game_Archive> ::MR_Receive(id);
			break;
		}
		Dbg("\tedited");
		int editor_index= id - msg_edited;
		int game_index= edited_games[editor_index].game_index;
		game_archive.Set(game_index,
				 edited_games[editor_index].game_editor.Get_Value());
		bool have_read= edited_games[editor_index].game_editor.Have_Read();
//		Update_List();
		Update_Game_Info(game_index);
		game_list.Set_Selected(game_index);
		Modify(); 
		if (have_read)
		{
			edited_games[editor_index].game_index= -1;
			int i= *edited_games - 1;  
			while (i >= 0 && 
			       edited_games[i].game_index == -1  &&
			       ! edited_games[i].game_editor.Is())
				--i;
			edited_games.Set(i + 1);
		}
		break;
	}

	case msg_insert:
	{
		int game_index= game_list.Get_Selected();
		if (game_index == -1)
			game_index= *game_archive;
		if (! game_archive.Insert_Game(game_index))
			break;
		for (int i= 0;  i < *edited_games;  ++i)
		{
			if (edited_games[i].game_index >= game_index)
				++ edited_games[i].game_index;
		}
		Update_Count();
		Update_List();
		game_list.Set_Selected(game_index);
		button_bar.Get(1).Set_Enabled(true);
		button_bar.Get(2).Set_Enabled(true); 
		Modify(); 
		break;
	}
		
	case msg_remove:
	{
		int game_index= game_list.Get_Selected();
		if (game_index == -1)
			break;
		for (int i= 0;  i < *edited_games;  ++i)
		{
			if (game_index == edited_games[i].game_index)
			{
				edited_games[i].game_editor.Close();
				edited_games[i].game_index= -1;
			}
			else if (edited_games[i].game_index > game_index)
				-- edited_games[i].game_index;
		}
		game_archive.Remove(game_index);
		Update_Count();
		Update_List();
		if (*game_archive != 0)
		{
			game_list.Set_Selected(game_index == *game_archive
					       ? game_index - 1
					       : game_index);
		}
		else
		{
			button_bar.Get(1).Set_Enabled(false);
			button_bar.Get(2).Set_Enabled(false); 
		}
		Modify(); 
		break;
	}

	case msg_edit:
	{
		Edit();
		break;
	}

	case msg_list_clicked:
		if (*game_archive == 0)
			break; 

		int r, c;
		game_list.Get_Clicked(r, c); 
		game_list.Set_Selected(r); 
		button_bar.Get(1).Set_Enabled(true);
		button_bar.Get(2).Set_Enabled(true); 
		if (game_list().Get_Double_Click())
			Edit(-1, false);
		break; 
	}
}

void
Game_Archive_Editor::Wdg_Close()
{
	Close_Editors();
	Document_Editor <Game_Archive> ::Wdg_Close();
}

void
Game_Archive_Editor::Close_Editors()
{
	for (int i= 0; i < *edited_games; ++i)
	{
		edited_games[i].game_editor.Close();
	}
	edited_games.Free();
}

void
Game_Archive_Editor::Edit(int index,
			  bool set_selection)
{
	/* Determine which game to edit. 
	 */
	Assert (index >= -1); 
	if (index == -1)
	{
		index= game_list.Get_Selected();
		if (index == -1)
			return;
	}
	else
	{
		if (set_selection)
			game_list.Set_Selected(index); 
	}

	/* Check whether game is already being edited. 
	 */
	for (int i= 0;  i < *edited_games;  ++i)
	{
		if (edited_games[i].game_index == index)
		{
			if (! game_archive.Check(index))
				return; 
			edited_games[i].game_editor.Open
				(msg_edited + i,
				 This,
				 game_archive[index]);
			return;
		}
	}

	if (! game_archive.Check(index))
		return;
	int size= *edited_games;
	if (! edited_games.Set(size + 1))
		return;
	edited_games[size].game_index= index;
	edited_games[size].game_editor.Open
		(msg_edited + size,
		 This,
		 game_archive[index]);
}

void Game_Archive_Editor::DE_New()
{
	game_archive.Clear();

	Update_List();
	Update_Count();
//	Modify(); 

//	return true; 
}

void Game_Archive_Editor::Update_Game_Info(int i)
{
	Dbg_Function();
	Dbg_Int(i);

	game_list.Set(0, i, game_archive.Get_Result(i));
	game_list.Set(1, i, game_archive.Get_White(i));
	game_list.Set(2, i, game_archive.Get_Black(i));
	game_list.Set(3, i, game_archive.Get_Event(i));
	game_list.Set(4, i, game_archive.Get_Round(i));
	game_list.Set(5, i, game_archive.Get_Date(i));
	game_list.Set(6, i, game_archive.Get_Site(i));
}
