#include "game_editor.hh"

#include "fen.hh"
//#include "get_selection_position.hh"
#include "keys.hh"
#include "game_stream.hh"
#include "ding.hh"
#include "dbg.hh"
#include "truncate.hh"
#include "rc.hh"
#include "format_last_move.hh"

bool
Game_Editor::Document_Editor_Open(Pointer <Packer> parent)
{
	persistent_notation.Open(This, msg_notation);

	if (! tool_bar.Open(parent()))
		return false; 

	tool_bar.Append(gl_new, msg_new, Trl("New game"));
	tool_bar.Append_Space();
	tool_bar.Append(gl_start, msg_start, Trl("To the start"));
	tool_bar.Append(gl_back, msg_back, Trl("Back"));
	tool_bar.Append(gl_forward, msg_forward, Trl("Forward"));
	tool_bar.Append(gl_end, msg_end, Trl("To the end"));
	tool_bar.Append_Space();
	tool_bar.Append(gl_run, msg_ev_run, Trl("Run engine"));
	tool_bar.Append(gl_pause, msg_ev_pause, Trl("Pause evaluator"));
	tool_bar.Append(gl_apply, msg_ev_play_best, Trl("Play best move"));

	if (! bevel_2.Open(parent()))
		return false; 

	parent().Set_Next_Resizeable(true);
	if (! middle_packer.Open(parent()))
		return false;

	middle_packer.Set_Next_Resizeable(false);
	if (! side_packer.Open(middle_packer))
		return false;

	Vector <String> titles;
	if (titles.Set(3))
	{
		titles[0]= "";
		titles[1]= "";
		titles[2]= "";
	}
	side_packer.Set_Next_Resizeable(true);
	if (! move_list.Open(side_packer, 
			     Multi_List_Box::Open_Info
			     (3,
			      scr_vertical,
			      msg_move_list,
			      *titles ? &titles : NULL))) 
		return false; 

	side_packer.Set_Next_Resizeable(true);
	if (! comment_frame.Open(side_packer,
				 Frame::Open_Info(ft_default, Trl("Comment"))))
		return false;
	comment_frame().Set_Next_Resizeable(true);
	if (! comment.Open(comment_frame(),
			   Multi_Edit::Open_Info
			   ("",
			    true)))
		return false;
//	comment.Set_Visible(false);
	show_comment= true;
	if (! comment_diagram.Open(comment_frame(), false,
				   Trl("Diagram")))
		return false;
	comment_frame.Set_Visible(false);

	middle_packer.Set_Next_Resizeable(true);
	if (! board.Open(middle_packer, 
			 game.Get_Position(), 
			 msg_dragged,
			 -1,
			 msg_board_turn))
		return false; 

	if (! status_bar.Open(parent(), 2))
		return false; 

	Update_Board();
	Update_Move_List();
	Update_Game_Info();

	return true; 
}

void 
Game_Editor::MR_Receive(int msg)
{
	Assert (Widget::Is());

	switch (msg)
	{
	default:
		Document_Editor <Game> ::MR_Receive(msg);
		break;

	case msg_dragged:
	{
		int from, to;
		if (! board().Get_Squares(from, to))
			break;

		if (to == 0)
		{
			board().Invalidate_Square(from);
			break;
		}

		int index= game.Get_Generator().Get_Square_Move(from, to);
		if (index >= 0)
			/* the move is legal */
		{
			if (game.Get_Generator()[index].promotion.Is())
				/* move is a promotion */
			{
				if (promotion_chooser.Is())
				{
					int s= promotion_chooser.Get_Selected();
					if (s != -1)
						index += s;
				}
			}

			Play(index);

			if (evaluator_widget.Widget::Is() &&
			    evaluator_widget.Get_Settings().auto_reply)
			{
				if (! Open_Evaluator_Widget())
					break;
				evaluator_widget.MR_Receive
					(Evaluator_Widget::msg_run);
			}
		}
		else /* no such legal move */
		{
			board().Invalidate_Square(from);
			board().Invalidate_Square(to); 

			/* If it was the last move in the game, take it back */ 

			if (game.Get_Overall_Location() != 0     &&
			    game.Get_Overall_Move(game.Get_Overall_Location() - 1)
			    .from == to   &&
			    game.Get_Overall_Move(game.Get_Overall_Location() - 1)
			    .to == from)
				Go_Backward();
		}

		break;
	}

	case msg_back:
		Go_Backward();
		break;

	case msg_forward:
		Go_Forward();
		break;

	case msg_start:
		Save_Comment();
		if (game.Go_To_Start())
		{
			Update_Board();
			Update_Move_List();
//			Set_Move_List_Selection(true);
			board().Invalidate_Position();
			Terminate_Evaluator();
		}
		break;

	case msg_end:
		Save_Comment();
		if (! game.Go_To_End())
			break;
		Update_Board();
		Set_Move_List_Selection(true);
		board().Invalidate_Position();
		Terminate_Evaluator();
		break;

	case msg_remove_end:
		game.Remove_End();
		Update_Move_List();
		Modify();
		break;

	case msg_new_position:
		position_editor.Open(msg_position_edited, 
				     This,
				     game.Get_Position());
		break;

	case msg_notation:
		Update_Move_List(); 
		Update_Legal_List(); 
		Update_Variations();
		break;

	case msg_edit_game_info:
		game_info_editor.Open(msg_game_info_edited,
				      This,
				      game.Get_Game_Info()); 
		break;

	case msg_game_info_edited:
		game.Set_Game_Info(game_info_editor.Get_Value());
		game_info_editor.Have_Read(); 
		Update_Game_Info();
		Modify();
		break; 

	case msg_position_edited:
		Terminate_Evaluator();
		Save_Comment();
		game.Open(position_editor.Get_Value());
		position_editor.Have_Read();
		Update_Board();
		Update_Move_List();
		board().Invalidate_Position();
		Modify();
		break;

	case msg_ev_play_best:
		if (evaluator_widget.Widget::Is() &&
		    evaluator_widget.Evaluator::Is())
		{
			int best_move= evaluator_widget.Get_Best_Move();
			if (best_move != -1)
				Play(best_move);
		}
		break;

	case msg_ev_play:
	{
 		if (*game.Get_Generator() == 0)
			break; 
		if (! evaluator_widget.Widget::Is())
			break;
		int best_move= evaluator_widget.Get_Best_Move();
		if (best_move != -1)
		{
			Play(best_move);
			Ding();
		}
		break;
	}

	case msg_ev_changed:
		if (show_evaluator_widget_on_start)
			evaluator_widget.Set_Visible(true);
		break;

	case msg_show_side_bar:
		side_packer.Set_Visible(! side_packer.Get_Visible());
		break;

	case msg_show_evaluator:
		if (! Open_Evaluator_Widget())
			break;
		evaluator_widget.Set_Visible(! evaluator_widget.Get_Visible()); 
		show_evaluator_widget_on_start= false; 
		if (evaluator_widget.Get_Visible())
			side_packer.Set_Visible(true); 
		break;

	case msg_show_legal_list:
		if (! legal_move_list.Is())
		{
			side_packer.Set_Next_Resizeable(true); 
			if (! legal_move_list.Open
			    (side_packer,
			     msg_legal))
				break;
			legal_move_list.Set_Visible(false); 
		}
		if (! legal_move_list.Get_Visible())
			legal_move_list.Set(game.Get_Generator(), 
					    persistent_notation.Get_Value()); 
		legal_move_list.Set_Visible(! legal_move_list.Get_Visible());
		if (legal_move_list.Get_Visible())
		{
			side_packer.Set_Visible(true); 
		}
		break;

	case msg_ev_run:
		if (! Open_Evaluator_Widget())
			break;
		if (evaluator_widget.Widget::Is() && 
		    *game.Get_Generator() != 0)
		{
			if (show_evaluator_widget_on_start)
				evaluator_widget.Set_Visible(true); 
			evaluator_widget.MR_Receive(Evaluator_Widget::msg_run);
		}
		break;

	case msg_ev_pause:
		if (evaluator_widget.Widget::Is())
			evaluator_widget.MR_Receive(Evaluator_Widget::msg_pause);
		break;

	case msg_move_list:
	{
		int row, col;
		move_list.Get_Clicked(row, col);
		if (row == 0 && col == 0)
		{
			MR_Receive(msg_start);
			break;
		}
		if (col == 0)
			break;
		/* overall index to go to. */
		int location= row * 2 + col;
		if (game.Get_First_Move() == cc_b)
			-- location;

		if (location == game.Get_Overall_Location())
			break;

		Save_Comment();
		game.Go_To_Overall_Location(location);
		Update_Board();
		Update_Move_List(); 
		Set_Move_List_Selection();
		board().Invalidate_Position();
		Terminate_Evaluator();
		break;
	}
		
	case msg_board_turn:
		if (material_widget.Is() && material_widget.Get_Visible())
			material_widget.Set(game.Get_Position(),
					 board().Get_Turned());
		break;

	case msg_show_comment:
		if (comment_frame.Get_Visible())
		{
			Save_Comment();
			comment_frame.Set_Visible(false);
			show_comment= false;
//			comment_diagram.Set_Visible(false);
		}
		else
		{
			Update_Comment();
			comment_frame.Set_Visible(true);
//			comment_diagram.Set_Visible(true);
		}
		break;
		
	case msg_legal:
	{
		int move= legal_move_list.Get_Selected();
		if (move == -1)
			break;
		Play(move);
		break;
	}

	case msg_variation_list:
	{
		Assert (variation_list().Is());
		int index= variation_list().Get_Selected();
		if (index == -1)
			break;
		bool can_leave= game.Get_Current_Location() == 0 &&
			game.Get_Variation_Depth() != 1;
		if (can_leave)
		{
			if (index == 0)
			{
				MR_Receive(msg_variation_leave); 
				break;
			}
			-- index;
		}

		if (index == 0)
		{
			Go_Forward();
			break;
		}
		
		Enter_Variation(index - 1);
		break;
	}
		
	case msg_variation_remove:
	{
		game.Remove_Variation();
		Update_Board();
		Update_Move_List();
		Update_Variations(); 
		board().Invalidate_Position(); 
		break; 
	}
	case msg_variation_leave:
		if (! game.Leave_Variation())
			break;
		Update_Variations();
		Update_Move_List();
		Update_Board(); 
		break;

	case msg_variation_make_parent:
		if (game.Make_Parent())
			Update_Move_List();
		break;
		
	case msg_variation_make_main:
		if (game.Make_Main())
			Update_Move_List(); 
		break;
	}
}

void
Game_Editor::Wdg_Close()
{
	Dbg_Function();
	Terminate_Evaluator();
	position_editor.Close();
	persistent_notation.Close();
	game_info_editor.Close(); 
	Save_Comment();
	Document_Editor <Game> ::Wdg_Close();
}

void
Game_Editor::Menued_Get(int menu,
			Menued::Entries &entries)
{
	switch (menu)
	{
	default:
		Document_Editor <Game> ::Menued_Get(menu, entries);
		break;

	case menu_main_ext:
		entries.Append_Submenu(menu_edit,
				       Trl("Edit"));
		entries.Append_Submenu(menu_view,
				    Trl("View"));
		entries.Append_Submenu(menu_moves,
				    Trl("Moves"));
		entries.Append_Submenu(menu_analyze,
				    Trl("Analyze"));
		break;

	case menu_edit:
		entries.Append_Command(msg_new_position,
				       Trl("Set up position"));
		entries.Append_Separator();
		entries.Append_Command(msg_edit_game_info,
				    Trl("Game information"));
		break;

	case menu_moves:
		entries.Append_Command(msg_start,
				       Trl("To the start"),
				       key_begin,
				       gl_start,
				       game.Get_Variation_Depth() != 0 ||
				       game.Get_Current_Location() != 0); 
		entries.Append_Command(msg_back,
				    Trl("Back"),
				    key_back,
				    gl_back);
		entries.Append_Command(msg_forward,
				    Trl("Forward"),
				    key_forward,
				    gl_forward);
		entries.Append_Command(msg_end,
				    Trl("To the end"),
				    key_end,
				    gl_end);
		entries.Append_Separator(); 
		entries.Append_Command(msg_remove_end,
				    Trl("Remove end of variation"),
				    key_remove_end);

//		entries.Append_Separator();
		entries.Append_Command(msg_variation_remove,
				       Trl("Remove variation"));
		entries.Append_Command(msg_variation_make_parent,
				       Trl("Reparent variation"));
		entries.Append_Command(msg_variation_make_main,
				       Trl("Reparent variation as main"));
		entries.Append_Command(msg_variation_leave,
				       Trl("Leave variation")); 
		break;
	
	case menu_analyze:
		if (! Open_Evaluator_Widget())
			break;
		entries.Set_Menued(evaluator_widget);
		evaluator_widget.Menued_Get(Evaluator_Widget::menu_main, entries);
		break;

	case menu_view:
		entries.Append_Checked(msg_show_side_bar,
				    Trl("Side bar"),
				    side_packer.Get_Visible(),
				    false);
		entries.Append_Checked(msg_show_legal_list,
				    Trl("Legal moves"),
				    legal_move_list.Is() &&
				    legal_move_list.Get_Visible(),
				    false); 
		entries.Append_Checked(msg_show_evaluator,
				    Trl("Evaluator process"),
				    evaluator_widget.Widget::Is() &&
				    evaluator_widget.Get_Visible(),
				    false); 
		entries.Append_Checked(msg_show_comment,
				    Trl("Comment"),
				    comment_frame.Get_Visible(),
				    false);
		entries.Append_Separator();
		entries.Set_Menued(persistent_notation);
		persistent_notation.Menued_Get
			(Persistent <Notation, Notation_Editor> ::menu_main,
			 entries);
		entries.Append_Separator();
		entries.Set_Menued(board());
		board().Menued_Get(Board::menu_main, entries);
		break;

	}
}

void
Game_Editor::Editor_Window_Set_Value(Pointer <const Game> value)
{
	if (value == NULL)
		game.Open();
	else
		game= value();
}

void
Game_Editor::Update_To_Move()
{
	if (! status_bar.Is(1))
		return;

	const char *text;
	if (*game.Get_Generator() == 0)
	{
		if (game.Get_Position().Is_Check())
			text= "Checkmate";
		else
			text= "Stalemate";
	}
	else
	{
		if (game.Get_Position().Is_Check())
			text= "Check";
		else
			if (game.Get_Position().Get_To_Move() == cc_w)
				text= "White to move";
			else
				text= "Black to move";
	}

	status_bar.Set
		(1,
		 Trl(text));
}

void
Game_Editor::Update_Last_Move()
{
	if (! status_bar.Is(0))
		return;

	String text;
	if (game.Get_Overall_Location() != 0)
	{
	  text += Format_Last_Move(game.Get_Current_Move_Name(game.Get_Current_Location() - 1),
				   game.Get_Current_Location(),
				   game.Get_Position().Get_To_Move(), 
				   persistent_notation());

#if 0
		String move_name= persistent_notation().Format_Move_Name
			(game.Get_Current_Move_Name(game.Get_Current_Location() - 1));

		text= Format("%d.%s %s",
			      (game.Get_Position().Get_Previous_Move_Count() - 1) / 2 + 1,
			      game.Get_Position().Get_To_Move() ? "" : "..",
			      move_name());
#endif /* 0 */ 
	}

	if (game.Get_Variation_Depth() == 1 &&
	    game.Get_Current_Location() == game.Get_Current_Length()  &&
	    game.Get_Game_Info().result != Game_Info::re_unfinished)
		/* If located at the end of the main variation. 
		 */
	{
		if (text.Is())
			text += " ";
		text += game.Get_Game_Info().Get_Result_Text();
	}

	status_bar.Set(0, text);
}

const Game &
Game_Editor::Editor_Window_Get_Value() 
{
	if (Is())
		Save_Comment();
	return game;
}

void Game_Editor::
Play(int move_index)
{
	Move move= game.Get_Generator()[move_index]; 

	/* move is next in current variation:  go forward */ 

	if (game.Get_Current_Length() > game.Get_Current_Location()     &&
	    (game.Get_Current_Move(game.Get_Current_Location())
	     == game.Get_Generator()[move_index]))
	{
		Go_Forward();
		return;
	}

	/* Move is a variation */ 

	if (game.Get_Current_Variation().Have_Variations(game.Get_Current_Location()))
	{
		for (int i= 0;  
		     i < *game.Get_Current_Variation().Get_Variations
			     (game.Get_Current_Location());
		     ++i)
		{
			const Variation &variation= game.Get_Current_Variation()
				.Get_Variations(game.Get_Current_Location())[i]();

			if (variation.Get_Length() != 0 &&
			    move == variation[0])
			{
//				Enter_Variation(i);
//				Go_Forward();
//				Save_Comment();
				Enter_Variation(i);
				return;
			}
		}
	}

	/* Play the move */

	Terminate_Evaluator();

	Move_Name move_name=
		game.Get_Generator().Get_Move_Name(move_index); 
	
	Save_Comment();
	if (game.Get_Current_Location() != game.Get_Current_Variation().Get_Length())
		game.Append_Variation();

	game.Append_Move(move_index);

	/* If the move ends the main variation with mate or stalemate,
	   change game info.  
	 */
	if ((move_name.check == 2 ||
	     move_name.check == 3)    &&
	    game.Get_Variation_Depth() == 1)
	{
		Game_Info::Result result;
		if (move_name.check == 3)
			result= 
				(game.Get_Position().Get_To_Move() == cc_w
				 ? Game_Info::re_white
				 : Game_Info::re_black);
		else if (move_name.check == 2)
			result= Game_Info::re_draw; 
		if (game_info_editor.Is_Editing())
		{
			game_info_editor().Set_Result(result); 
		}
		else
		{
			game.Get_Game_Info().result= result; 
		}
	}

	board().Invalidate_Move_Squares(move);

	String move_text= persistent_notation().Format_Move_Name
		(game.Get_Current_Move_Name(game.Get_Current_Location() - 1));

	Update_Move_List();

	Update_Board();
	Modify();
}

bool
Game_Editor::Editor_Window_Change_Value(const Game &new_game)
{
	game= new_game;
	Terminate_Evaluator();
	Update_Move_List();
	Update_Board();
	board().Invalidate_Position();
	Modify();
	return true;
}

bool Game_Editor::
Editor_Window_Try_Load(String file_name, 
		       Pointer <Game> game)
{
	Position position;
	if (Read_From_File(file_name, position))
	{
		game().Open(position);
		return true;
	}
	else
		return false;
}

void 
Game_Editor::Update_Legal_List()
{
	if ((! legal_move_list.Is()) ||
	    ! legal_move_list.Get_Visible())
		return;

	legal_move_list.Set(game.Get_Generator(), persistent_notation());
}

void Game_Editor::
Update_Board()
{
	Dbg_Function();

	/* Last move on board. 
	 */ 
	Update_Board_Selection();
#if 0
	Dbg_Int(game.Get_Variation_Depth());
	Dbg_Int(game.Get_Current_Location()); 
	if (game.Get_Variation_Depth() == 1 && game.Get_Current_Location() == 0)
		board().Clear_Selection();
	else
	{
		const Move &move= game.Get_Overall_Move
			(game.Get_Overall_Location() - 1);
		board().Set_Selection(move.from, move.to);
	}
#endif /* 0 */ 
	Update_Last_Move();
	Update_To_Move();
	Update_Legal_List(); 

	if (Material_Widget::Get_Show(game.Get_Position()))
	{
		if (! material_widget.Is())
			material_widget.Open(side_packer);
		material_widget.Set(game.Get_Position(), board().Get_Turned());
		material_widget.Set_Visible(true);
	}
	else
	{
		if (material_widget.Is())
			material_widget.Set_Visible(false);
	}

	Update_Comment();

	bool have_promotion= game.Get_Generator().Get_Flags() & gn_promotion; 
	if (have_promotion)
	{
	  if (promotion_chooser.Is() ||
	      promotion_chooser.Open(side_packer, persistent_notation()))
	    promotion_chooser.Set_Used(true);
	}
	else
	{
		if (promotion_chooser.Is())
			promotion_chooser.Set_Used(false);
	}

	Update_Variations();
}

void
Game_Editor::DE_New()
{
	Terminate_Evaluator();

	game.Open();

	Update_Board();

	Update_Move_List();
	board().Invalidate_Position();
}

void Game_Editor::
Update_Move_List()
{
	/* Fill game move list with overall variation. 
	 */

	Chess_Color color= game.Get_First_Move();
	int moves= game.Get_Overall_Length();
	moves += color;
	int row_count= (moves + 1) / 2;
	if (row_count == 0)  row_count= 1;
	move_list.Set_Row_Count(row_count);

	/* Add move numbers
	 */
	int first= game.Get_Initial_Position().Is()
		? game.Get_Initial_Position()().Get_Previous_Move_Count() / 2 + 1
		: 1;
	for (int i= 0;  i < row_count;  ++i)
		move_list.Set(0, i, Format("%d.", i + first));
	
	/* Add "..." if black begins. 
	 */
	if (color)
	{
		move_list.Set(1, 0, "...");
	}

	/* Add "" and "" if overall game is empty. 
	 */
	if (moves == 0)
	{
		move_list.Set(1, 0, "");
		move_list.Set(2, 0, "");
	}

	/* Add moves
	 */
	for (int i= 0;  i < game.Get_Overall_Length();  ++i)
	{
		String text= persistent_notation().Format_Move_Name(game.Get_Overall_Move_Name(i));
		if (game.Is_Overall_Variation_Move(i))
			text += '*';
		move_list.Set
			((i + color) % 2 + 1,
			 (i + color) / 2,
			 text);
	}

	/* Add "" if last move is white. 
	 */
	if (moves % 2)
		move_list.Set(2, row_count - 1, "");

	Set_Move_List_Selection(true);
}

void Game_Editor::
Set_Move_List_Selection(bool scroll)
{
	if (game.Get_Overall_Location() > 0)
	{
		int index= game.Get_Overall_Location()
			+ (game.Get_First_Move() == cc_b ? 1 : 0) - 1; 
		move_list.Set_Selected(index / 2, scroll);
	}
	else
		move_list.Set_Selected(0, scroll);
}

void
Game_Editor::Go_Backward()
{
	Save_Comment();

	if (game.Get_Overall_Location() == 0)
		return; 

	Move move= game.Get_Overall_Move(game.Get_Overall_Location() - 1);

	if (! game.Go_Backward())
		return;

	Update_Board();
	board().Invalidate_Move_Squares(move); 
	Update_Move_List();

	Terminate_Evaluator();

	board().Invalidate_Move_Squares(game.Get_Current_Move(game.Get_Current_Location())); 
}

void
Game_Editor::Save_Comment()
{
	if (! comment_frame.Get_Visible())
		return;

	String text= comment().Get_Text();
	if (comment_diagram.Get_Checked())
		text= String('\04') + text;
	game.Set_Overall_Comment(text);
}

void Game_Editor::
Update_Comment()
{
	String text= game.Get_Overall_Comment();
	if (! text.Is())
	{
		if (comment_frame.Get_Visible())
		{
			comment().Set_Text("");
			comment_diagram.Set_Checked(false);
		}
		return;
	}
	if (! comment_frame.Get_Visible())
	{
		if (show_comment)
		{
			comment_frame.Set_Visible(true);
//			comment_diagram.Set_Visible(true); 
		}
		else
			return;
	}

	if (text[0] == '\04')
	{
		text= text.Sub(1, *text - 1);
		comment_diagram.Set_Checked(true);
	}
	else 
		comment_diagram.Set_Checked(false);
	
	comment().Clear();
	comment().Append_Text(text); 
}

void Game_Editor::
Update_Variations()
{
	if (variation_list.Is())
		variation_list().Set_Row_Count(0);
	bool show_list= false;

	/* Leave variation
	 */
	int current_location= game.Get_Current_Location();
	if (current_location == 0 && game.Get_Variation_Depth() != 1)
	{
		if (! Open_Variation_List())
			return;
		variation_list().Set_Row_Count(1);
		variation_list().Set
			(0, 0, "..");
		variation_list().Set
			(1, 0, Trl("Leave variation")); 
		show_list= true;
	}

	/* Main continuation. 
	 */
	if (current_location == *game.Get_Current_Variation())
		goto end;

	if (! Open_Variation_List())
		return;

	variation_list().Set_Row_Count(variation_list().Get_Row_Count() + 1);
	variation_list().Set
		(0, variation_list().Get_Row_Count() - 1,
		 persistent_notation().Format_Move_Name
		 (game.Get_Current_Variation()
		  .Get_Move_Name(current_location)));
	variation_list().Set
		(1, variation_list().Get_Row_Count() - 1,
		 Trl("Main variation"));

	/* Subvariations
	 */
	if (game.Get_Current_Variation().Have_Variations(current_location))
	{
		int variation_count= 
			*game.Get_Current_Variation()
			.Get_Variations(current_location);
		if (variation_count != 0)
			show_list= true;
		variation_list().Set_Row_Count
			(variation_list().Get_Row_Count() + variation_count);
		for (int i= 0;  
		     i < variation_count;
		     ++i)
		{
			String text= persistent_notation().Format_Move_Name(
				game.Get_Current_Variation()
				.Get_Variations(current_location)()[i]()
				.Get_Move_Name(0));
			variation_list().Set
				(0,
				 i + variation_list().Get_Row_Count() - variation_count, 
				 text + '*');
		}
	}

 end:

	if (variation_list.Is())
		variation_list.Set_Visible(show_list); 
}

void Game_Editor::
Go_Forward()
{
	Save_Comment();
	if (! game.Go_Forward())
		return;

	Update_Board();
	Set_Move_List_Selection(true);
	Terminate_Evaluator();

	board().Invalidate_Move_Squares(
		game.Get_Overall_Move(game.Get_Overall_Location() - 1)); 
}

void Game_Editor::Update_Game_Info()
{
	Dbg_Function();

	String white= game.Get_Game_Info().white;
	if (white.Is())
		white= Truncate(white, 15);
	else
		white= Trl("White");
	Dbg_Str(white());
	move_list.Set_Title(1, white);

	String black= game.Get_Game_Info().black;
	if (black.Is())
		black= Truncate(black, 15);
	else
		black= Trl("Black");
	Dbg_Str(black());
	move_list.Set_Title(2, black);
}

bool Game_Editor::Open_Variation_List()
{
	if (variation_list.Is())
		return true;

//	left_packer.Set_Next_Resizeable(true); 
	return variation_list.Open
		(side_packer,
		 Frame::Open_Info(ft_default, Trl("Variations")),
		 Multi_List_Box::Open_Info
		 (2, scr_vertical, msg_variation_list)); 
}

bool Game_Editor::Open_Evaluator_Widget()
{
	if (evaluator_widget.Widget::Is())
		return true; 
	if (! evaluator_widget.Open(side_packer, 
				    msg_ev_play, 
				    msg_ev_changed,
				    msg_ev_play_best,
				    game.Get_Position()))
		return false;
	show_evaluator_widget_on_start= true; 
	evaluator_widget.Set_Visible(false); 
	return true; 
}

void Game_Editor::Terminate_Evaluator()
{
	if (evaluator_widget.Widget::Is())
		evaluator_widget.Terminate();
}

void Game_Editor::Enter_Variation(int index)
{
	Save_Comment();
	game.Play_Variation(index);
	Update_Move_List();
	Set_Move_List_Selection(true);
	Update_Variations();
	Update_Board();
}

void Game_Editor::Update_Board_Selection()
{
	Dbg_Int(game.Get_Variation_Depth());
	Dbg_Int(game.Get_Current_Location()); 
	if (game.Get_Variation_Depth() == 1 && game.Get_Current_Location() == 0)
		board().Clear_Selection();
	else
	{
		const Move &move= game.Get_Overall_Move
			(game.Get_Overall_Location() - 1);
		board().Set_Selection(move.from, move.to);
	}
}

