#ifndef GAME_EDITOR_INCLUDED
#define GAME_EDITOR_INCLUDED

#include "document_editor.hh"
#include "game.hh"
#include "clean_board.hh"
#include "tool_bar.hh"
#include "status_bar.hh"
#include "position_editor.hh"
#include "evaluator_widget.hh"
#include "game_stream.hh"
#include "material_widget.hh"
#include "game_info_editor.hh"
#include "multi_list_box.hh"
#include "legal_move_list.hh"
#include "notation_stream.hh"
#include "persistent.hh"
#include "promotion_chooser.hh"

class Game_Editor
	:  public Document_Editor <Game> 
{
public:

	const Game &Editor_Window_Get_Value();

	static const char *Get_Help()
		{
			return "Chess game editor.\n";
		}

private:

	enum {msg_start= Document_Editor <Game> ::msg_custom,
	      msg_forward, msg_back, msg_end, 
	      msg_new_position, msg_remove_end, 
	      msg_position_edited, msg_dragged, 
	      msg_show_side_bar, msg_ev_play_best,
	      msg_ev_play,
	      msg_ev_run, msg_ev_pause, msg_ev_changed,
	      msg_move_list, msg_board_turn,
	      msg_show_evaluator, 
	      msg_edit_game_info, msg_game_info_edited,
	      msg_show_legal_list, msg_show_comment,
	      msg_legal, msg_variation_list,
	      msg_variation_remove, 
	      msg_variation_leave,
	      msg_variation_make_parent,
	      msg_variation_make_main, 
	      msg_notation,
	};

	enum {menu_edit= Document_Editor <Game> ::menu_custom,
	      menu_view, menu_moves,
	      menu_analyze}; 

	Game game;
	bool show_evaluator_widget_on_start;
	bool show_comment;
	Persistent <Notation, Notation_Editor> persistent_notation;

	Tool_Bar tool_bar;
	Promotion_Chooser promotion_chooser;
	Bevel bevel_2;
	H_Packer middle_packer;
	V_Packer side_packer;
	Evaluator_Widget evaluator_widget;
	Material_Widget material_widget;
	Clean_Board board;
	Status_Bar status_bar;
	Multi_List_Box move_list;
	Legal_Move_List legal_move_list;
	Framed <V_Packer> comment_frame;
	Gtk_Scrolled <Multi_Edit> comment;
	Check_Box comment_diagram; 
	Framed <Multi_List_Box> variation_list;

	Owned_Editor <Position_Editor> position_editor;
	Owned_Editor <Game_Info_Editor> game_info_editor;

	void Wdg_Close();

	void MR_Receive(int);

	void Menued_Get(int menu, Menued::Entries &entries);

	bool Document_Editor_Open(Pointer <Packer>);

	const char *EW_Type_Name() const 
		{
			return Trl("Chess Game"); 
		}
	const char *EW_Short_Type_Name() const
		{
			return Trl("Game"); 
		}

	void Editor_Window_Set_Value(Pointer <const Game>);
	int Editor_Window_Use()
		{
			return use_ok;
		}
	bool Editor_Window_Change_Value(const Game &);
	bool Editor_Window_Try_Load(String, Pointer <Game>);
	void DE_New();

	void Update_To_Move();
	void Update_Last_Move();
	void Update_Move_List();
	/* Also sets move list selection. 
	 */
	void Update_Legal_List(); 
	void Play(int move_index);
	void Update_Board();
	/* Do everything when position has changed, except updating
	   the board.  Don't update the move list and variation
	   list. 
	 */
	void Set_Move_List_Selection(bool scroll_to_visible= false);
	/* Set selection in move list to reflect last played move. 
	 */
	void Go_Forward();
	void Go_Backward();
	void Save_Comment();
 	void Update_Comment();
	void Update_Variations();
	void Update_Game_Info();
	bool Open_Variation_List();
	bool Open_Evaluator_Widget();
	void Terminate_Evaluator();
	void Enter_Variation(int variation_index);
	void Update_Board_Selection();
};

#endif /* ! GAME_EDITOR_INCLUDED */
