#include "generator.hh"

#include "simple_notation.hh"

bool
Generator::New(const Position &new_position)
{
	/* Generate legal moves 
	 */
	Position tmp_position= new_position;
	moves.Free();
	Generate_Legal(tmp_position, moves, true, false);

	flags= gen_flags;

	/* Generate move names
	 */
	if (! names.Set(*moves))
		return false;

	position= new_position;
	bool d, dl, dr;

	for (int i= 0;  i < *moves;  ++i)
	{
		names[i].piece_type= tmp_position.Get_Piece(moves[i].from).Get_Type();
		Piece piece= tmp_position.Get_Piece(moves[i].to);
		names[i].capture= piece.Is() ? 1 + piece.Get_Type() : 0;
		names[i].from_file= Square_File(moves[i].from);
		names[i].from_rank= Square_Rank(moves[i].from);
		names[i].to_file= Square_File(moves[i].to);
		names[i].to_rank= Square_Rank(moves[i].to);
//		names[i].variation= 0;

		if (moves[i].promotion.Is())
			names[i].special= 3 + moves[i].promotion.Get_Type();
		else if (moves[i].ep_square)
		{
			names[i].special= 1;
			names[i].capture= 1;
		}
		else if (moves[i].castling)
			names[i].special= moves[i].castling % 2 + 2;
		else
			names[i].special= 0;
      
		if (tmp_position.Get_Piece(moves[i].from).Get_Type() != pt_P)
		{
			d= false; dl= false; dr= false;
			for (int j= 0;  j < *moves;  ++j)
			{
				if (j != i                     &&
				    moves[i].to == moves[j].to   &&
				    (tmp_position.Get_Piece(moves[i].from) ==
				     tmp_position.Get_Piece(moves[j].from)))
				{
					d= true;
					if (Square_File(moves[i].from)==Square_File(moves[j].from))
						dl= true;
					if (Square_Rank(moves[i].from)==Square_Rank(moves[j].from))
						dr= true;
				}
			}
			if (d)
			{
				if (dl)
				{
					if (dr)
						names[i].precision= 3;
					else
						names[i].precision= 2;
				}
				else
					names[i].precision= 1;
			}
			else
				names[i].precision= 0;
		}

		names[i].check= 0;
		tmp_position << moves[i];
		if (tmp_position.Is_Check())
			names[i].check += 1;
		Vector <Move> tmp_moves;
		Generate_Legal(tmp_position, tmp_moves, true, true);
		if (*tmp_moves == 0)
			names[i].check += 2;
		tmp_position >> moves[i];
	}

	return true; 
}

const Move &
Generator::Get_Move(int i) const
{
	Assert (Is() && i >= 0 && i < *moves);
	return moves[i];
}

Move_Name
Generator::Get_Move_Name(int i) const
{
	Assert (Is() && i >= 0 && i < *moves);
	return names[i];
}

int Generator::Get_Square_Move(int from, int to) const
{
	for (int i= 0;  i < Get_Count();  ++i)
	{
		if (from == Get_Move(i).from && to == Get_Move(i).to)
			return i; 
	}
	return -1;

#if 0
	bool found= false;
	int i= 0, count= Get_Count();
	while (i < count && ! found)
	{
		if (from == Get_Move(i).from && to == Get_Move(i).to)
			found= true;
		++i;
	}
	return found ? i-1 : -1;
#endif /* 0 */ 
}

int
Generator::Get_e2e4_Notation_Move(String move_text) 
	const
{
	if (move_text[0] < 'a' || move_text[0] > 'h'  ||
	    move_text[1] < '1' || move_text[1] > '8'  ||
	    move_text[2] < 'a' || move_text[2] > 'h'  ||
	    move_text[3] < '1' || move_text[3] > '8')
		return -1;

	int
		from=  Make_Square(move_text[0] - 'a', move_text[1] - '1'),
		to=    Make_Square(move_text[2] - 'a', move_text[3] - '1');
  
	int index= Get_Square_Move(from, to);

	switch (move_text[4])
	{
	case 'n':  case 'N':  index += 3;  break;
	case 'b':  case 'B':  index += 2;  break;
	case 'r':  case 'R':  index += 1;  break;
	}

	if (index >= Get_Count())
		return -1;

	return index; 
}

String
Generator::Get_Simple_Move_Name(int index)
	const
{
	return Simple_Notation(Get_Move(index));
}

void
Generator::Free()
{
	Assert (Is());
	position= NULL;
}
