#include "get_dir.hh"

#include "file_name.hh"
#include "main.hh"
#include "dbg.hh"
#include "file_test.hh"
#include "version_number.hh"
#include "format.hh"
#include "get_home.hh"

#include <limits.h> 

String
Get_Dir(Install_Dir id)
{
	Dbg_Function();

	static String base_dir;
	/* Base installation directory.  Eg: `/usr', `/home/joe'... 
	 */

	if (! base_dir.Is())
	{
		Dbg_Str(Main_Get_Name()());

		String program_name= Main_Get_Name(); /* argv[0] */ 

		if (program_name[0] != '/') 
			/* argv[0] is no absolute file name */ 
		{
			/* Try to find out absolute file name */

			/* Read `/proc/PID/exe'.  Works only on Linux. 
			 */
			char buffer[PATH_MAX + 1];
			String file_name= Format("/proc/%d/exe", (int)getpid()); 
			Dbg_Str(file_name());
			int r= readlink(file_name(), buffer, PATH_MAX);
			if (r != -1)
			{
				program_name= String(buffer, r);
				Dbg_Str(program_name());
			}
			else
			{
				String file_name("/usr/bin/", program_name);
				if (File_Is_Readable(file_name))
				{
					base_dir= "/usr";
				}
				else
				{
					file_name= Get_Home() / "bin" / program_name;
					if (File_Is_Readable(file_name))
						base_dir= Get_Home(); 
					else
						base_dir= "."; 
				}
			}
		}
		if (! base_dir.Is())
			base_dir= Path_Name(Path_Name(program_name));

		Dbg_Str(base_dir());
	}

	switch (id)
	{
	default:
		Assert_Unreached();
		return "";

	case id_bin:
	{
		static String bin_dir;
		if (! bin_dir.Is())
			bin_dir= base_dir / "bin";
		return bin_dir;
	}

	case id_share:
	case id_copying:
	{
		static String share_dir;
		if (! share_dir.Is())
		{
			share_dir= base_dir / "share";
			String baby_dir= share_dir / "baby"; 
			if (File_Is_Dir(baby_dir))
				share_dir= baby_dir / VERSION_NUMBER "/share"; 
		}
		Dbg_Str(share_dir());
		if (id == id_share)
			return share_dir;
		static String copying_name;
		if (! copying_name.Is())
		{
			String name= share_dir / "COPYING";
			Dbg_Str(name());
			if (File_Is_Readable(name))
			{
				Dbg("\treadable");
				copying_name= name;
			}
			else
			{
				Dbg("\tnot readable"); 
				copying_name= share_dir / ".." / "COPYING"; 
			}
		}
		Dbg_Str(copying_name());
		return copying_name; 
	}
	}
}
