#ifndef GTK_OBJECT_INCLUDED
#define GTK_OBJECT_INCLUDED

#include "assert.hh"
#include "pointer.hh"
#include "gtk.hh"
#include "canvas.hh"
#include "stretchness.hh"
#include "mouse_action.hh"
#include "mouse_button.hh"
#include "key.hh"
#include "message_receiver.hh"

#include <gtk/gtk.h>

class Gtk_Object
//	:  public Message_Receiver
	:  virtual public Pointed
{
public:

	Gtk_Object()
		:  gtkwidget(NULL)
		{
		}
	virtual ~Gtk_Object()
		{
			Assert (gtkwidget == NULL);
		}

	virtual void Wdg_Close()
		{
		}
 	void Attach(GtkWidget *new_gtkwidget); 

	void Close(bool force= false); 

	bool Is() const
		{
			return gtkwidget != NULL;
		}

	GtkWidget *Get() const
		{
			return gtkwidget;
		}

	Point Get_Size() const
		{
			Assert (Is());

			Point result (gtkwidget->allocation.width,
				      gtkwidget->allocation.height);

			return result;
		}

	virtual void Wdg_Do_Close()
		{
			gtk_widget_destroy(gtkwidget);
			Assert (gtkwidget == NULL); 
		}
	virtual void Wdg_Destroy()
		{
		}
	virtual void Wdg_Get_Size_Request(int & /* width */ ,
					  int & /* height */ ,
					  bool & /* resizeable */ )
		/* Return preferred size, and whether widget can be
		   resized.  The widget may check width and height.
		   If one of them is not -1, then the returned value
		   should not be bigger than that value. 
		 */
		{
//			width= height= 1;
//			resizeable= true;
		}

	virtual void Wdg_Mouse(Mouse_Action, 
			       Mouse_Button,
			       Point,
			       bool)
		{
		}
	virtual void Wdg_Mouse_Cursor()
		{
		}
	virtual void Wdg_Draw(Canvas &)
		{
		}
	virtual void Wdg_Key_Press(Key)
	  {
	  }
	virtual bool Wdg_Close_Query()
	  /* Whether the widget can be closed.  Called before Wdg_Close(). 
	   */
	  {
			return true; 
		}

	void Connect_Mouse_Signals(GtkWidget *gtkwidget);
	void Connect_Key_Signals(GtkWidget *gtkwidget);

private:

	GtkWidget *gtkwidget;
	bool force_close;
	/* Set by Close().  Read by Signal_Delete(). 
	 */
	bool deleted;
	/* TRUE if Signal_Delete() was called. 
	 */

	virtual void Wdg_Gtk_Delete()
		/* When Signal_Delete() is received.  Only called on
		   windows. 
		*/
		{
		}
	virtual void Wdg_Size(Point)
		{
		}

	/* To do:  check return values of signals handlers. 
	 */
	static gint Signal_Delete(GtkWidget *this_gtkwidget,
				  GdkEvent  * /* event */ ,
				  gpointer   data); 
	/* Called before destruction of an object.  Only
	   called for windows.   
	*/


	static void Signal_Destroy(GtkWidget *,
				   gpointer   data); 
	/* Called after destruction. 
	 */

	static void Signal_Expose(GtkWidget *this_gtkwidget,
				  GdkEventExpose *event,
				  gpointer data); 
	static void Signal_Size_Allocate(GtkWidget *,
					 GtkAllocation *,
					 gpointer);
	static void Signal_Size_Request(GtkWidget *,
					GtkRequisition *,
					gpointer);
	static gint Signal_Motion_Notify_Event(GtkWidget *,
					       GdkEventMotion *,
					       gpointer);
	static gint Signal_Button_Press_Event(GtkWidget *,
					      GdkEventButton *,
					      gpointer);
	static gint Signal_Button_Release_Event(GtkWidget *,
					      GdkEventButton *,
					      gpointer);
	static gint Signal_Key_Press_Event(GtkWidget *,
					   GdkEventKey *,
					   gpointer);
};

#endif /* ! GTK_OBJECT_INCLUDED */
