#ifndef ICS_INCLUDED
#define ICS_INCLUDED

/* ICS (internet chess server) client. 
 */

#include "socket_child_process.hh"
#include "position.hh"
#include "message_receiver.hh"
#include "generator.hh"

class ICS
	:  private Socket_Child_Process
{
public:

	ICS() 
		{
		  position.Reset();
		}

	bool Open(String host_name,
		  int port,
		  String user,
		  String passwd);
	bool Is() const 
	  /* connection established */ 
	  {
	    return Socket_Child_Process::Is();  
	  }

	bool Get_Inside_Game() const
	  {
	    Assert (Is());
	    return inside_game; 
	  }

	void Close() 
		{
			Socket_Child_Process::Close();
		}

	Stream &Get_Output() {return Socket_Child_Process::Get_Output();	} 
	Stream &Get_Error_Output()
	  {
	    return Socket_Child_Process::Get_Error_Output(); 
	  }

	const Position &Get_Position() const 
	  {
	    return position;
	  }

	bool Proceed();
	
	String Get_User() const
	  {
			Assert (Is());
			return user;
		      }

	String Get_Host() const
		{
			Assert (Is());
			return host;
		      }

	void Seek(int minutes,
		  int secs);
	bool User_Play(int from, int to);
	/* Whether move was played. 
	 */
	Chess_Color Get_Playing_Side() const
		{
			Assert(Is());
			Assert(inside_game);
			return observing ? cc_w : play_color;
		}
	String Get_White() const
		{
		  Assert (Is() && inside_game);
		  return white_player;
		}
	String Get_Black() const
		{
		  Assert (Is() && inside_game);
			return black_player;
		}
	int Get_Result() const
		{
		  return result;
		}

	void Resign()
		{
			Assert (Is() && inside_game);
			Write("resign");
		}

	void Put(String s) 
	  {
	    Write(s);
	  }
	String Get_Error() const
		{
		  return error;
		}

	const Generator &Get_Generator() const
	  {
	    return generator; 
	  }

private:

	void IOCP_Output(String line);
	void IOCP_Error_Output(String line);

	Position position;
	Generator generator; 
	bool ready; /* user & passwd were written */ 
	String host;
	String user;
	String passwd;
	bool seeking;
	/* A game is seeked or must be seeked. 
	 */
	int seek_m, seek_s;
	bool inside_game;
	/* Whether we're currently playing a game. 
	 */
	bool observing;
	String white_player, black_player;
	/* Set when GAME is TRUE. */
	Chess_Color play_color;

	int result; /* 0=black, 1= draw, 2= white, 3= aborted */
	String error;

	void Write_Seek();
	/* Output the seek command. 
	 */
	int Style_12(const char *p);
	/* Get a style-12 string whithout the "<12>".  Return 0 on
	   success. 
	*/
	void Write(String, bool echo= true);

	virtual void ICS_Changed()= 0;
	/* board changed */
	virtual void ICS_New_Game()= 0;
	virtual void ICS_Finished()= 0;
	virtual void ICS_Playing(String move)= 0;
	/* an opponent's move */
	virtual void ICS_Message(String message)= 0;
	/* Whithout trailing newline. 
	 */
	virtual void ICS_Clock(int white, int black)= 0;
	/* seconds. 
	 */
};

#endif /* ! ICS_INCLUDED */
