#include "io_child_process.hh"

#include "read_line.hh"
#include "error.hh"
#include "pipe.hh"
#include "dbg.hh"

bool
IO_Child_Process::Open(String program_file_name,
		       const Vector <String> &args,
		       bool nice)
{
	Assert (! Is());

	/* Remote end of the pipe.  Closed in this process after the
	   fork. 
	 */
	Stream in_rem, out_rem, err_rem;

	if (in.Is())
		in.Close();
	if (out.Is())
		out.Close();
	if (err.Is())
		err.Close();

	if (! (Pipe(in,      in_rem)  &&
	       Pipe(out_rem, out)     &&
	       Pipe(err_rem, err)))
		return false;

	if (! Child_Process::Open(program_file_name,
				  args,
				  & in_rem,
				  & out_rem,
				  & err_rem,
				  nice))
		return false;

	in.Set_Block(false);
	out.Set_Block(false);
	err.Set_Block(false);

	return true;
}

void 
IO_Child_Process::Pause()
{
	Assert (Is());
	Child_Process::Pause();
}

bool
IO_Child_Process::Proceed()
{
	bool success;
	bool close= false;
	
	do 
	{
		String line;
		success= Read_Line(out, line);
		if (success)
		{
			IOCP_Output(line);
		}
	}
	while (success);

	if (out.End_Of_File())
	{
		Dbg("\tout.End_Of_File() ==> close.");
		close= true;
	}		

	if (Error::Get_Code() != EAGAIN)
	{
		Dbg("\terror ==> close.");
		close= true;
	}

	do 
	{
		String line;
		success= Read_Line(err, line);
		if (success)
		{
			IOCP_Error_Output(line);
		}
	}
	while (success);

	if (err.End_Of_File())
	{
		Dbg("\terr.End_Of_File() ==> close.");
		close= true;
	}		

	if (Error::Get_Code() != EAGAIN)
	{
		Dbg("\terror ==> close.");
		close= true;
	}

	return close;
}

void
IO_Child_Process::Write(String line)
{
	Write_Formatted(Get_Input(),
			"%s\n",
			line());
}
