#ifndef IO_CHILD_PROCESS_INCLUDED
#define IO_CHILD_PROCESS_INCLUDED

/* Child process with built-in redirection of IO. 
 */
/* "Input" and "output" are from the viewpoint of the child process.  
 */

/* Proceed() must be called when data is available on output or error
   output. 
 */

#include "child_process.hh"
#include "stream.hh"

class IO_Child_Process
	:  private Child_Process
{
public:

	virtual ~IO_Child_Process() 
		{
		}

	bool Is() const
		{
			return Child_Process::Is();
		}

	bool Open(String program_name,
		  const Vector <String> &args,
		  bool nice= false);
	/* Must be closed. */

	Stream &Get_Input() 
		{
			return in; 
		}
	Stream &Get_Output() 
		{
			return out; 
		}
	Stream &Get_Error_Output() 
		{
			return err; 
		}
	/* The child is finished when end-of-file is encountered on
	   one of the outputs.  All files are set to non-blocking.  If
	   encoutering EOF on reading, call Close(). 
	*/
	void Close()
		{
			in.Close();
			out.Close();
			err.Close();
			Child_Process::Close();
		}

	void Pause();
	bool Paused() const
		{
			return Child_Process::Paused();
		}

	bool Proceed();
	/* Return TRUE if child process has finished, but don't close
	   if finished.  Call the virtual functions. 
	*/

	void Write(String line);
	/* Write a line to STDIN of process.  '\n' is added to LINE. 
	 */
	Process_Usage Get_Process_Usage() const
		{
			return Child_Process::Get_Process_Usage();
		}
	Process Get_Process() const
		{
			return Child_Process::Get_Process(); 
		}

private:

	Stream in, out, err;
	/* All set to non-blocking when opened. 
	 */

	/* Whithout the terminating new-line.  A line was read.
	   Called by Proceed(). 
	*/
	virtual void IOCP_Output(String /* line */ )
		{
		}
	virtual void IOCP_Error_Output(String /* line */ )
		{
		}
};

#endif /* ! IO_CHILD_PROCESS_INCLUDED */
