#include "legal_move_list.hh"

#include "trl.hh"
#include "format.hh"

void Legal_Move_List::
Set(const Generator &generator,
    const Notation &notation)
{
	move_list.Set_Text(Format(*generator == 1 
				  ? Trl("One legal move")
				  : Trl("%d legal moves"), 
				  *generator));
	move_list().Set_Row_Count(*generator);
	for (int i= 0;  i < *generator;  ++i)
	{
		move_list().Set
			(0, i,
			 notation.Format_Move_Name
			 (generator.Get_Move_Name(i)));
	}
//	const char *const names[4]= {"wK", "wO-O-O", "bO-O", "bO-O-O"};
	String castlings;
	for (int i= 0;  i < 4;  ++i)
	{
		bool possible= ! 
			(generator.Get_Position().Get_Moved(1 + i / 2 * 3) ||
			 generator.Get_Position().Get_Moved(2 + i / 2 * 3 - (i % 2 * 2)));
		if (possible)
		{
			if (castlings.Is())
				castlings += ", ";
			castlings += (i / 2 ? 'b' : 'w');
			castlings += (notation.Get_Pieces()[5 - i % 2]); 
		}
	}
	if (castlings.Is())
	{
		castlings_text.Set_Text(Format("%s: %s",
					       Trl("Castlings"),
					       castlings()));
	}
	else
		castlings_text.Set_Text(Trl("No castlings"));

	String hm= Format("%s:  %d",
			  Trl("Draw clock"),
			  generator.Get_Position().Get_Halfmove_Clock() / 2);
	if (generator.Get_Position().Get_Halfmove_Clock() >= 100)
	{
		hm += " (";
		hm += Trl("draw");
		hm += ")"; 
	}

	hm_text.Set_Text(hm);
}

bool Legal_Move_List::
Open(Pointer <Widget> parent,
     int new_msg_clicked)
{
	msg_clicked= new_msg_clicked;

	if (! V_Packer::Open(parent))
		return false;

	castlings_text.Open(This);
	hm_text.Open(This);

	V_Packer::Set_Next_Resizeable(true); 

	if (! move_list.Open
		(This,
		 Multi_List_Box::Open_Info
		 (1,
		  scr_vertical,
		  msg_list_clicked)))
		goto close;

	return true;

 close:
	V_Packer::Close();
	return false; 
}

void Legal_Move_List::MR_Receive(int msg)
{
	if (msg == msg_list_clicked)
		MR_Send(msg_clicked);
	else
		V_Packer::MR_Receive(msg);
}

