#ifndef MENUED_INCLUDED
#define MENUED_INCLUDED

#include "glyph_type.hh"
#include "pointer.hh"
#include "string.hh"
#include "key.hh"
#include "vector.hh"
#include "message_receiver.hh"

/* A MENUED can be asked the content of a menu.  
 */

class Menued
	:  virtual public Pointed,
	   virtual public Message_Receiver
{
public:

	class Entry
	/* Info about one menu item
	 */
	{
	public:

		enum Type {type_command, 
			   type_sub,
			   type_separator};


		Pointer <Menued> target;
		/* Where to send the command to.
		 */
		Type type;
		/* Either a command or a submenu. 
		 */
		int id;
		/* Either the command ID or the submenu ID. 
		 */
		String text;

		Key key;
		/* Key for that command.  Menues can receive key commands. 
		 */
		Glyph_Type glyph_type;
		bool enabled;

		Entry(Pointer <Menued> new_target,
		      int new_id,
		      String new_text, 
		      Key new_key= Key(Key::no_key),
		      Glyph_Type new_glyph_type= gl_default,
		      bool new_enabled= true)
			/* Command
			 */
			:  target(new_target),
			   type(type_command),
			   id(new_id),
			   text(new_text),
			   key(new_key),
			   glyph_type(new_glyph_type),
			   enabled(new_enabled)
			{
			}
		Entry(Pointer <Menued> new_target,
		      int new_id,
		      String new_text,
		      bool new_checked,
		      int new_kind,
		      Key new_key= Key(Key::no_key),
		      bool new_enabled= true)
			/* For checked menues.
			 * KIND=  0:  check box
			 *        1:  radio box
			 */
			:  target(new_target),
			   type(type_command), 
			   id(new_id),
			   text(new_text),
			   key(new_key),
			   glyph_type(new_checked ? (new_kind == 1
						     ? gl_radio
						     : gl_check)
				      : gl_nothing),
			   enabled(new_enabled)
			{
			}
		Entry()
			/* separator */
			:  type(type_separator)
			{
			}
		Entry(Pointer <Menued> new_target,
		      int new_id,
		      String new_text,
		      bool new_enabled= true)
			/* submenu
			 */
			:  target(new_target),
			   type(type_sub),
			   id(new_id),
			   text(new_text),
			   key(Key(Key::no_key)),
			   glyph_type(gl_default),
			   enabled(new_enabled)
			{
			}
	};

	class Entries
		:  public Vector <Entry>
	{
	public:

		void Append_Command(int msg, 
				    String text,
				    Key key= Key(Key::no_key),
				    Glyph_Type glyph_type= gl_default,
				    bool enabled= true);

		void Append_Checked(int msg,
				    String text,
				    bool checked,
				    bool radio= false,
				    bool enabled= true)
			{
				Append(Entry(menued, msg, text,
					     checked, radio ? 1 : 0, enabled));
			}

		void Append_Separator()
			{
				Append(Entry());
			}

		void Append_Submenu(int menu,
				    String text,
				    bool enabled= true)
			{
				Append(Entry(menued, menu, text, enabled));
			}

		void Set_Menued(Menued &new_menued)
			{
				menued= & new_menued;
			}

		void Free()
			{
				menued= NULL;
				Vector <Entry> ::Free(); 
			}
	private:

		Pointer <Menued> menued;

	};

	virtual ~Menued()
		{
		}

	virtual void Menued_Get(int /* menu */ , 
				Entries & )
		{
		}
};

#endif /* ! MENUED_INCLUDED */
