#include "multi_list_box.hh"

#include "dbg.hh"

bool
Multi_List_Box_Content::Open(Pointer <Widget> parent,
			     int new_column_count,
			     Scroll_Direction /* scr */,
			     int new_msg_clicked,
			     Pointer <const Vector <String> > titles)
{
	column_count= new_column_count; 

	row= -1;
	row_count= 0; 

	GtkWidget *gtkwidget= NULL; 

	const gchar *text[column_count];
	for (int i= 0;  i < column_count;  ++i)
		text[i]= titles.Is() 
			?(gchar*)titles()[i]()
			: "";
	gtkwidget= gtk_clist_new_with_titles
		(column_count,
		 (gchar **) text); 

	if (gtkwidget == NULL)
		return false;

	for (int i= 0;  i < column_count;  ++i)
		gtk_clist_set_column_auto_resize 
			(GTK_CLIST(gtkwidget),
			 i,
			 TRUE);

	if (titles == NULL)
		gtk_clist_column_titles_hide(GTK_CLIST(gtkwidget)); 

	if (! Widget::Open(parent, gtkwidget))
		return false;

	msg_clicked= new_msg_clicked; 

	gtk_clist_column_titles_passive (GTK_CLIST (Get()));
	gtk_signal_connect(GTK_OBJECT(Get()),
			   "select-row",
			   GTK_SIGNAL_FUNC(& Signal_Select_Row),
			   this);

	return true;
}

void Multi_List_Box_Content::Signal_Select_Row
(GtkCList * /* clist */ ,
 gint new_row,
 gint new_column,
 GdkEventButton *event,
 gpointer data)
{
	Dbg_Function();

	Multi_List_Box_Content &box= Deref((Multi_List_Box_Content *)data);

	box.row= new_row;
	box.column= new_column; 

	if (event == NULL)
		return;

	box.double_click= Deref(event).type == GDK_2BUTTON_PRESS; 
	box.MR_Send(box.msg_clicked);
}

void Multi_List_Box_Content::
Set_Row_Count(int new_row_count)
{
	Dbg_Function();
	Dbg_Int(new_row_count);

//	int row_count= next_column == 0 ? next_row : next_row + 1;

	Dbg_Int(row_count);
//	Dbg_Int(next_row);
//	Dbg_Int(next_column);

	if (new_row_count > row_count) /* add rows */
	{
		gchar *text[column_count];
		for (int i= 0;  i < column_count;  ++i)
			text[i]= "";
		for (int i= row_count;  i < new_row_count;  ++i)
		{
			gtk_clist_append(GTK_CLIST(Get()),
					 text); 
		}
	}
	else if (new_row_count < row_count) /* remove rows */ 
	{
		for (int i= row_count - 1;  i >= new_row_count;  --i)
			gtk_clist_remove(GTK_CLIST(Get()),
					 i);
		if (row >= new_row_count)
			row= -1;
	}

	row_count= new_row_count;
}

void
Multi_List_Box_Content::Set_Selected(int new_row)
{
	row= new_row;
	gtk_clist_select_row(GTK_CLIST(Get()),
			     row,
			     0);
}

void
Multi_List_Box::Set_Selected(int row, bool scroll)
{
	Dbg_Function();

	This().Set_Selected(row); 

	if (! scroll)
		return; 

	GtkAdjustment *a= gtk_scrolled_window_get_vadjustment
		(GTK_SCROLLED_WINDOW(Get()));

	if (a == NULL)
		return;

	float min_visible= Deref(a).value;
	float max_visible= Deref(a).value + Deref(a).page_size;
	Dbg("\t%f ... %f",
	    min_visible, max_visible);
	
	float new_ideal_value_min=
		Deref(a).lower 
		+ ((Deref(a).upper - Deref(a).lower) 
		   * row 
		   / This().Get_Row_Count());
	float new_ideal_value_max=
		Deref(a).lower 
		+ ((Deref(a).upper - Deref(a).lower) 
		   * (row + 1)
		   / This().Get_Row_Count());
	Dbg("\tideal= %f ... %f", 
	    new_ideal_value_min,
	    new_ideal_value_max);
		
	if (new_ideal_value_min >= min_visible &&
	    new_ideal_value_max <= max_visible)
		return;

	gtk_adjustment_set_value
		(GTK_ADJUSTMENT(a),
		 new_ideal_value_min);

	gtk_adjustment_value_changed(a);
}

bool Multi_List_Box::Open(Pointer <Widget> parent,
			  Open_Info open_info)
{
	return Gtk_Scrolled
		<Multi_List_Box_Content> ::Open
		(parent, Gtk_Scrolled
		 <Multi_List_Box_Content>
		 ::Open_Info(Scroll_Widget::Open_Info
			     (open_info.scroll_direction), 
			     open_info));
}
