#ifndef PERSISTENT_INCLUDED
#define PERSISTENT_INCLUDED

#include "rc.hh"
#include "keys.hh"
#include "menued.hh"

template <class Type, class Type_Editor>
class Persistent
	:  public Menued,
	   public Widget /* must be Message_Receiver */ 
{
public:

	enum {menu_main}; 

	~Persistent()
		{
		}

	void Open(Widget &new_owner,
		  int new_msg_changed)
		{
			Dbg_Function();
			owner= new_owner;
			msg_changed= new_msg_changed;
			bool loaded= RC_Load(object);
			Dbg_Int(loaded);
			if (! loaded)
			{
				Dbg("\tloading");
				object.Reset();
			}
		}

	void Close()
		{
			editor.Close(); 
			owner= NULL; 
		}

	const Type &Get_Value() const
		{
			return object; 
		}
	const Type &operator () () const
		{
			return Get_Value(); 
		}
	Type &Get_Change_Value() 
		/* The value can be changed, but Changed() must be
		   called after the change. 
		 */
		{
			return object;
		}

	void Menued_Get(int menu, 
			Entries &entries)
		{
			if (menu == menu_main)
			{
				Type_Editor *p= NULL;
				entries.Append_Command
					(msg_edit,
					 Trl(p->Type_Editor::EW_Type_Name()),
					 key_settings); 
			}
		}
	const Type &Get_Old_Value() const
		/* Only called inside MSG_CHANGED. 
		 */
		{
			return old_object();
		}
	void Changed()
		{
			RC_Save(object);
		}

private:

	enum {msg_edit= Widget::msg_custom, msg_edited, msg_custom};

	Type object;
	Dynamic <Type> old_object; 
	Owned_Editor <Type_Editor> editor;
	int msg_changed;
	Pointer <Widget> owner; 

	void MR_Receive(int msg)
		{
			switch (msg)
			{
			default:
				Widget::MR_Receive(msg);
				break;

			case msg_edit:
				editor.Open(msg_edited,
					    This,
					    object,
					    owner()); 
				
				break;
				
			case msg_edited:
				if (! old_object.Alloc(object))
					break;
				object= editor.Get_Value();
				owner().MR_Receive(msg_changed);
				old_object.Free();
				editor.Have_Read(); 
				RC_Save(object);
				break;
			}
		}	       

};

#endif /* ! PERSISTENT_INCLUDED */
