#include "pipe.hh"

bool
Pipe(Stream &fa,
     Stream &fb,
     int type)
{
	Assert (! fa.Is());
	Assert (! fb.Is());
	Assert (type == Stream::text || type == Stream::binary);

	int fd[2];

	if (0 != pipe(fd))
		return false;

	FILE *i= fdopen(fd[1], "w");
	if (i == NULL)
	{
		int save_errno= errno;
		close(fd[0]);
		close(fd[1]);
		errno= save_errno;
		return false;
	}

	FILE *j= fdopen(fd[0], "r");
	if (j == NULL)
	{
		int save_errno= errno;
		fclose(i);
		close(fd[0]);
		errno= save_errno;
		return false;
	}

	fa.Set_FILE(i);
	fb.Set_FILE(j);

	Stream_Call_Hook(fd[0]);
	Stream_Call_Hook(fd[1]);
	
	if (type == Stream::text)
	{
		setvbuf(i,
			NULL,
			_IOLBF,
			0x80);
		setvbuf(j,
			NULL,
			_IOLBF,
			0x80);
	}
	else
	{
		setvbuf(i, NULL, _IONBF, 0);
		setvbuf(j, NULL, _IONBF, 0);
	}

	return true;
}

