#include "position.hh"

#include "moved.hh"
#include "moves.hh"

void
Position::Init_Outside()
{
	/* Above & below.
	 */
	for (int i= 0;  i < 2;  ++i)
	{
		for (int j= i * 100;  j < i * 100 + 20; ++j)
			pieces[j]= pi_outside;
	}

	/* Left & right. 
	 */
	for (int i= 20; i < 100;  i += 10)
	{
		pieces[i]= pi_outside; 
		pieces[i + 9]= pi_outside; 
	}
}

void
Position::Reset()
{
	Piece_Type first_row[]=
	{
		pt_R, pt_N, pt_B, pt_Q, pt_K, pt_B, pt_N, pt_R
	};

	/* empty squares */
	for (int i= 41; i<81; i+=10)
	{
		int j= i+8;
		for (int k= i; k<j; ++k)
			pieces[k]= pi_empty; 
	}


	/* white pawns */
	for (int i= 31; i<39; ++i)
		pieces[i]= pi_wP; 

	/* black pawns */
	for (int i= 81; i<89; ++i)
		pieces[i]= pi_bP; 

	/* First rows.
	 */
	for (int i= 0; i<2; ++i)
	{
		int j= 21+i*70;
		for (int l= 0; l<8; ++l)
			pieces[j+l]= Piece((Chess_Color)i, first_row[l]);
	}

	/* Material
	 */
	for (int cc= 0;  cc < 2;  ++cc)
	{
		material(pt_P, cc)= 8;
		material(pt_N, cc)= 2;
		material(pt_B, cc)= 2;
		material(pt_R, cc)= 2;
		material(pt_Q, cc)= 1;
		material(pt_K, cc)= 1;
	}

	/* Other 
	 */
	for (int i= 0;  i < 6;  ++i)
		moved[i]= 0;
	to_move= cc_w;
	ep_square= 0;
	king_squares[cc_w]= sq_e1;
	king_squares[cc_b]= sq_e8;
	previous_move_count= 0;
	halfmove_clock= 0;
}

void
Position::Clear()
{
	for (int i= 21; i < 101; i += 10)
	{
		int j= i+8;
		for (int k= i; k < j; ++ k)
			pieces[k]= Piece(false);
	}
	material.Clear();
	to_move= cc_w;
	previous_move_count= 0; /* == to_move */ 
	halfmove_clock= 0;
//	moved[0]= 1;
//	moved[2]= 1;
//	moved[3]= 1;
//	moved[5]= 1;
}

void
Position::operator << (const Move &move)
{
	Assert (pieces[move.from].Is());
	Assert (move.ep_square != 0 || pieces[move.to] == move.captured_piece);
	Assert (move.to > 0 && move.to < 120);

	/* Move the piece */

	pieces[move.to]= pieces[move.from];
	pieces[move.from]= Piece(false);

	/* Change side to move */

	to_move= CC_Turn(to_move);

	/* Update MOVED */ 

	if (Is_Move_Square(move.from))
		++ moved[Square_Move_Index(move.from)];

	if (Is_Move_Square(move.to))
		++ moved[Square_Move_Index(move.to)];

	/* MATERIAL & HALFMOVE_CLOCK */

	if (move.captured_piece.Is())
	{
		-- material(move.captured_piece.Get_Type(), to_move);
		halfmove_clock= 0;
	}
	else
		++ halfmove_clock;

	if (pieces[move.to].Get_Type() == pt_P)
		halfmove_clock= 0;

	/* Castling */

	if (move.castling != 0)
	{
		switch (move.castling)
		{
		case Move::w_O_O:
			pieces[sq_h1]= Piece(false);
			pieces[sq_f1]= Piece(cc_w, pt_R);
			break;
  
		case Move::w_O_O_O:
			pieces[sq_a1]= Piece(false);
			pieces[sq_d1]= Piece(cc_w, pt_R);
			break;
  
		case Move::b_O_O:
			pieces[sq_h8]= Piece(false);
			pieces[sq_f8]= Piece(cc_b, pt_R);
			break;

		case Move::b_O_O_O:
			pieces[sq_a8]= Piece(false);
			pieces[sq_d8]= Piece(cc_b, pt_R);
			break;

		default:
			Assert_Unreached();
		}
	}

	/* Promotion */ 

	else if (move.promotion.Is())
	{
		pieces[move.to]= move.promotion;
		-- material(pt_P, CC_Turn(to_move));
		++ material(move.promotion.Get_Type(), CC_Turn(to_move));
	}

	/* En passant capture */

	else if (move.ep_square != 0)
		pieces[move.ep_square]= Piece(false);

	/* New en passant square for double pawn advances */ 

	if (pieces[move.to] == Piece(cc_w, pt_P) && 
	    Square_Rank(move.from) == sq_2       &&
	    Square_Rank(move.to) == sq_4)
		ep_square= move.to;
	else if (pieces[move.to] == Piece(cc_b, pt_P) &&
		 Square_Rank(move.from) == sq_7       &&
		 Square_Rank(move.to) == sq_5)
		ep_square= move.to;
	else
		ep_square= 0;

	/* KING_POS */

	if (pieces[move.to].Is_Piece_Of_Type(pt_K))
	{
		king_squares[pieces[move.to].Get_Color()]= move.to; 
	}

	/* PREVIOUS_MOVES */

	++ previous_move_count;

	Assert (material.Check()); 
}

void Position::operator >> (const Move &move)
{
	Assert(pieces[move.to].Is());

	/* PIECES */ 

	pieces[move.from]= pieces[move.to];
	pieces[move.to]= move.captured_piece;

	/* TO_MOVE */ 

	to_move= CC_Turn(to_move);

	/* MOVED */ 

	if (Is_Move_Square(move.from))
		-- moved[Square_Move_Index(move.from)];
	if (Is_Move_Square(move.to))
		-- moved[Square_Move_Index(move.to)];

	/* old EP_SQUARE */ 

	ep_square= move.ep_poss_square;

	/* MATERIAL */

	if (move.captured_piece.Is())
	{
		++ material(move.captured_piece.Get_Type(), CC_Turn(to_move));
	}

	/* HALFMOVE_CLOCK */

	halfmove_clock= move.halfmove_clock; 
	Assert (halfmove_clock >= 0);
	
	/* CASTLING */ 

	if (move.castling != 0)
	{
		switch (move.castling)
		{
		case Move::w_O_O:
			pieces[sq_f1]= Piece(false);
			pieces[sq_h1]= Piece(cc_w, pt_R);
			break;

		case Move::w_O_O_O:
			pieces[sq_d1]= Piece(false);
			pieces[sq_a1]= Piece(cc_w, pt_R);
			break;

		case Move::b_O_O:
			pieces[sq_f8]= Piece(false);
			pieces[sq_h8]= Piece(cc_b, pt_R);
			break;

		case Move::b_O_O_O:
			pieces[sq_d8]= Piece(false);
			pieces[sq_a8]= Piece(cc_b, pt_R);
			break;

		default:
			Assert (0);
		}
	}

	/* Take back en passant move */

	else if (move.ep_square != 0)
	{
		Assert(ep_square == move.ep_square);
		pieces[move.ep_square]= move.captured_piece;
		pieces[move.to]= Piece(false);
	}

	/* Take back promotion */ 

	else if (move.promotion.Is())
	{
		pieces[move.from]= Piece(to_move, pt_P);
		++ material(pt_P, to_move);
		-- material(move.promotion.Get_Type(), to_move);
	}

	/* KING_SQUARE */ 

	if (pieces[move.from].Is_Piece_Of_Type(pt_K))
		king_squares[pieces[move.from].Get_Color()]= move.from;

	/* PREVIOUS_MOVES */ 

	Assert (previous_move_count > 0);
	--previous_move_count;

	Assert (material.Check());
}

bool
Position::Square_Attacked(int square, 
			  Chess_Color color)
	const 
{
	/* king */
	for (int offs= 0;  offs < 8;  ++ offs)
		if (Get_Piece(square + moves_offset[offs]) == Piece(CC_Turn(color), pt_K)) 
			return true;

	/* pawn */
	if (color == cc_b)
	{
		if (Get_Piece(square -  9) == Piece(cc_w, pt_P))
			return true;
		if (Get_Piece(square - 11) == Piece(cc_w, pt_P))
			return true;
	}
	else
	{
		if (Get_Piece(square +  9) == Piece(cc_b, pt_P))
			return true;
		if (Get_Piece(square + 11) == Piece(cc_b, pt_P))
			return true;
	}

	/* files and ranks */
	for (int offs= 0;  offs < 4;  ++ offs)
	{
		int i= 1;

		while (Get_Piece(square + i * moves_offset[offs]).Is_Empty())
			++i;

		if (Get_Piece(square + i * moves_offset[offs]) == Piece(CC_Turn(color), pt_Q)   ||
		    Get_Piece(square + i * moves_offset[offs]) == Piece(CC_Turn(color), pt_R))
			return true;
	}

	/* diagonals */
	for (int offs= 4;  offs < 8;  ++ offs)
	{
		int i= 1;
      
		while (Get_Piece(square + i * moves_offset[offs]).Is_Empty())
			++i;

		if (Get_Piece(square + i * moves_offset[offs]) == Piece(CC_Turn(color), pt_Q) ||
		    Get_Piece(square + i * moves_offset[offs]) == Piece(CC_Turn(color), pt_B))
			return true;
	}

	/* knight */
	for (int offs= 8;  offs < 16;  ++ offs)
		if (Get_Piece(square + moves_offset[offs]) == Piece(CC_Turn(color), pt_N)) 
			return true;

	return false;
}

void Position::Update_Material()
{
	for (int cc= 0;  cc < 2;  ++cc)
		for (int pt= 0;  pt < 6;  ++pt)
			material(pt, cc)= 0;

	for (int file= sq_a;  file <= sq_h;  ++file)
		for (int rank= sq_1;  rank <= sq_8;  ++rank)
		{
			Piece piece= Get_Piece(Make_Square(file, rank));
			if (piece.Is())
			{
				++ material(piece.Get_Type(), piece.Get_Color());
			}
		}
}

void Position::Transfer_Piece(int from, int to)
{
	Assert (Is_Inside_Square(from));
	Assert (Is_Inside_Square(to));

	Piece to_p= pieces[to];
	if (to_p.Is())
		-- material(to_p.Get_Type(),
			    to_p.Get_Color());
	pieces[to]= pieces[from];
	pieces[from]= Piece(false);
	if (pieces[to].Get_Type() == pt_K)
		king_squares[pieces[to].Get_Color()]= to; 
}

void Position::Change_To_Move()
{
	to_move= ! to_move; 
	previous_move_count += (-1 + 2 * to_move);
}

void Position::Insert_Piece(int square, Piece piece)
{
	Assert (Is_Inside_Square(square));
	Assert (piece.Is());
	Assert (! pieces[square].Is());
	pieces[square]= piece;
	++ material(piece.Get_Type(), piece.Get_Color()); 
	if (piece.Get_Type() == pt_K)
		king_squares[piece.Get_Color()]= square;
}


