#include "position_editor.hh"

#include "trl.hh"
//#include "get_selection_position.hh"
#include "check_position.hh"
#include "inform.hh"
#include "read_fen.hh"
#include "piece_name.hh"
#include "keys.hh"
#include "fen.hh"

bool
Position_Editor::Document_Editor_Open(Pointer <Packer> parent)
{
	Dbg_Function();
	
	Dbg_Int(Get_Modified());

	Normalize_Castlings();

	Dbg_Int(Get_Modified());

	if (! tool_bar.Open(parent()))
		return false;

	tool_bar.Append(gl_new, msg_new, Trl("Reset position"));
	tool_bar.Append(gl_clear, msg_clear, Trl("Clear position"));

	if (! bevel.Open(parent()))
		return false;
	Dbg_Int(Get_Modified());
	parent().Set_Next_Resizeable(true); 
	if (! middle_packer.Open(parent()))
		return false;

	if (! side_packer.Open(middle_packer))
		return false;

	clear_button.Open(side_packer, 
			  msg_clear,
			  Trl("Clear board"));
	reset_button.Open(side_packer,
			  Document_Editor
			  <Position> ::msg_new,
			  Trl("Reset board"));

	if (! color_chooser.Open(side_packer, 
				 msg_color,
				 Trl("To move"),
				 2))
		return false; 

	Dbg_Int(Get_Modified());
	if (! (color_chooser.Append(Trl("White"))  &&
	       color_chooser.Append(Trl("Black"))))
		return false;
	color_chooser.Set_Selected(position.Get_To_Move());

	if (! new_type.Open(side_packer, -1, Trl("Type of piece to add"), 2))
		return false;

	for (int i= 0; i < 12; ++i)
	{
		new_type.Append(Format("%s%s",
				       i / 6 ? Trl("w") : Trl("b"),
				       Trl(Piece_Name(i % 6))));
	}

	ep_chooser.Open(side_packer, 
			msg_ep,
			Trl("Last move"),
			1);
	Dbg_Int(Get_Modified());
	ep_chooser.Set_Visible(false);

	if (! castling_widget.Open(side_packer,
				   Frame::Open_Info(ft_default,
						    Trl("Possible castlings"))))
		return false;

	if (! castling_check_boxes[0].Open(castling_widget(),
					   false,
					   "wO-O",
					   msg_castling))
		return false;
	if (! castling_check_boxes[1].Open(castling_widget(),
					   false,
					   "wO-O-O",
					   msg_castling_1))
		return false;
	if (! castling_check_boxes[2].Open(castling_widget(),
					   false,
					   "bO-O",
					   msg_castling_2))
		return false;
	if (! castling_check_boxes[3].Open(castling_widget(),
					   false,
					   "bO-O-O",
					   msg_castling_3))
		return false;

	if (! pm_packer.Open(side_packer))
		return false;
	pm_packer.Set_Next_Resizeable(true);
	if (! pm_text.Open(pm_packer, Trl("First move number:")))
		return false;
	if (! pm_edit.Open(pm_packer, 
			   Int_Edit::Open_Info(msg_pm,
					       1, 
					       1, 10000)))
		return false;

	if (! hm_packer.Open(side_packer))
		return false;
	hm_packer.Set_Next_Resizeable(true); 
	if (! hm_text.Open(hm_packer, Trl("Halfmove clock:")))
		return false;
	if (! hm_edit.Open(hm_packer,
			   Int_Edit::Open_Info(msg_hm,
					       0,
					       0, 
					       100)))
		return false;

	middle_packer.Set_Next_Resizeable(true); 
	if (! board.Open(middle_packer,
			 position,
			 msg_dragged,
			 msg_remove))
		return false;

	if (! fen_packer.Open(parent()))
		return false;
	fen_packer.Set_Visible(false);
	if (! fen_text.Open(fen_packer, "FEN:"))
		return false;
	fen_packer.Set_Next_Resizeable(true);
	if (! fen_edit.Open(fen_packer))
		return false;
	if (! fen_button.Open(fen_packer, 
			      msg_fen,
			      Trl("Set")))
		return false;
	if (! status_bar.Open(parent(), 1))
		return false; 

	persistent_notation.Open(This, msg_notation);

	Dbg_Int(Get_Modified());
	Update_Board();
	Dbg_Int(Get_Modified());

	return true; 
}

void 
Position_Editor::MR_Receive(int msg)
{
	Dbg_Function();
	Dbg_Int(msg);

	switch (msg)
	{
	default:
		Document_Editor <Position> ::MR_Receive(msg);
		break;

	case msg_remove:
	{
		int from, to;
		if (! board().Get_Squares(from, to))
			break;
		Remove_Piece(from);
		break;
	}

	case msg_dragged:
	{
		int from, to;
		if (board().Get_Squares(from, to))
		{
			if (position.Get_Piece(from).Is())
				/* dragged a piece */
			{
				if (to == 0) /* drag out of the board */
				{
					Remove_Piece(from);
				}
				else if (to != from)
				{
					position.Transfer_Piece(from, to);
					Update_Legality(); 
					Update_EP();
					Update_FEN();
					Update_Castlings();
					Update_Legal();
					Modify();
				}
			}
			else /* add new piece */
			{
				Insert_New_Piece(from);
			}
			board().Invalidate_Square(from);
			if (to != 0)
				board().Invalidate_Square(to);
		}
		break;
	}

	case msg_clear:
		position.Clear();
		Update_Board(); 
		Modify();
		break;

	case msg_color:
	{
		Chess_Color new_to_move= 
			color_chooser.Get_Chooser().Get_Selected() == 0
			? cc_w : cc_b; 
		if (position.Get_To_Move() == new_to_move)
			break;
		position.Change_To_Move();
		Update_Legality();
		Update_EP();
		Update_FEN();
		Update_Legal();
		Modify();
		break;
	}

	case msg_fen:
	{
		Dbg("\tmsg_fen");
		Position new_position;
		if (! Read_FEN(fen_edit.Get_Text(), new_position))
		{
			Inform(Trl("FEN error"), 
			       Error::Get()); 
			break;
		}
		position= new_position;
//		position.Update_Material();
		Update_Board();
		break;
	}

	case msg_ep:
	{
		Dbg("\tmsg_ep");
		int sel= ep_chooser.Get_Selected();
		Dbg_Int(sel);
		if (sel < 0)
			break;
		if (sel == ep_chooser.Get_Count() - 1)
		{
			position.Get_EP_Square()= 0;
		}
		else
		{
			Assert (sel < ep_chooser.Get_Count());
			int file= ep_index[sel];
			position.Get_EP_Square()= Make_Square
				(file, 
				 position.Get_To_Move() == cc_w ? sq_5 : sq_4);
		}
		Dbg_Int(position.Get_EP_Square());
		Update_Legality();
		Update_FEN();
		Update_Legal();
		Modify();

		Assert(legal);
		/* If the ep chooser could be clicked, then the
		   position could be clicked, and the new position
		   with another last move must also be legal. 
		*/

		break;
	}

	case msg_show_legal:
		if (! legal_move_list.Is())
		{
			side_packer.Set_Next_Resizeable(true);
			if (! legal_move_list.Open(side_packer))
				break;
		}
		else
			legal_move_list.Set_Visible(! legal_move_list.Get_Visible());
		if (legal_move_list.Get_Visible())
			side_packer.Set_Visible(true);
		Update_Legal();
		break;

	case msg_show_fen:
		fen_packer.Set_Visible(! fen_packer.Get_Visible()); 
		if (fen_packer.Get_Visible())
			side_packer.Set_Visible(true); 
		break;

	case msg_castling:
	case msg_castling_1:
	case msg_castling_2:
	case msg_castling_3:
	{
		int index= msg - msg_castling;
		Assert (index >= 0 && index < 4);
		int moved= castling_check_boxes[index].Get_Checked() ? 0 : 1;
		position.Get_Moved(Castling_Moved(index))= moved;
		/* legality doesn't change. */
		/* EP doesn't change. */
		Update_FEN();
		Update_Legal();
		Modify();
		break;
	}

	case msg_show_side_bar:
		side_packer.Set_Visible(! side_packer.Get_Visible());
		break;

	case msg_hm:
		position.Set_Halfmove_Clock(hm_edit.Get_Value()); 
		Update_FEN(); 
		break;

	case msg_pm:
		Retrieve_PM(); 
		Update_FEN();
		break;

 case msg_notation:
		Update_Legal(); 
		break;

	}
}

int
Position_Editor::Editor_Window_Use()
{
	return legal ? use_ok : use_retry;
}

void
Position_Editor::Menued_Get(int menu,
			    Menued::Entries &entries)
{
	switch (menu)
	{
	default:
		Document_Editor <Position> ::Menued_Get(menu, entries);
		break;

	case menu_main_ext:
		entries.Append_Submenu(menu_edit,
				       Trl("Edit"));
		entries.Append_Submenu(menu_view,
				       Trl("View"));

		break;

	case menu_edit:
		entries.Append_Command(msg_clear,
				       Trl("Clear board"));
		break;

	case menu_view:
		entries.Append_Checked(msg_show_side_bar,
				       Trl("Side bar"),
				       side_packer.Get_Visible(),
				       false);
		entries.Append_Checked(msg_show_legal,
				       Trl("Legal moves"),
				       legal_move_list.Is() &&
				       legal_move_list.Get_Visible(),
				       false);
		entries.Append_Checked(msg_show_fen,
				       Trl("FEN notation"),
				       fen_packer.Get_Visible(),
				       false);
		entries.Append_Separator();
		entries.Set_Menued(persistent_notation);
		persistent_notation.Menued_Get
			(Persistent <Notation, Notation_Editor> ::menu_main,
			 entries);
		entries.Append_Separator();
		entries.Set_Menued(board());
		board().Menued_Get(Board::menu_main, entries);
		break;
	}
}

void
Position_Editor::Update_Board()
{
	board().Invalidate_Position();
	color_chooser.Get_Chooser().Set_Selected
		(position.Get_To_Move() == cc_w ? 0 : 1); 
	Update_Legality(); 
	Update_EP();
	Update_FEN();
	Update_Castlings();
	Update_PM();
	Update_HM();
	Update_Legal();
}

void Position_Editor::Editor_Window_Set_Value(Pointer <const Position> value)
{
	if (value == NULL)
		position.Reset();
	else
		position= value();
	Normalize_Castlings();
	position.Update_Material(); 
}

const Position &
Position_Editor::Editor_Window_Get_Value() 
{
	if (Is())
		Retrieve_PM();
	
	if (position[sq_a1] != pi_wR)
		position.Get_Moved(0)= 1;
	if (position[sq_e1] != pi_wK)
		position.Get_Moved(1)= 1;
	if (position[sq_h1] != pi_wR)
		position.Get_Moved(2)= 1;
	if (position[sq_a8] != pi_bR)
		position.Get_Moved(3)= 1;
	if (position[sq_e8] != pi_bK)
		position.Get_Moved(4)= 1;
	if (position[sq_h8] != pi_bR)
		position.Get_Moved(5)= 1;

	return position;
}

bool
Position_Editor::Editor_Window_Change_Value(const Position &p)
{
	position= p;
	position.Update_Material();
	Update_Board();
	return true;
}

void
Position_Editor::DE_New()
{
	position.Reset();
	Update_Board();
}

void Position_Editor::Update_Legality()
{
	Dbg_Function();

	int n= Check_Position(position);
	if (n == nrm_ok)
	{
		Dbg("\tlegal");
		status_bar.Set(0, Trl("Position is legal"));
		legal= true;
	}
	else
	{
		Dbg("\tnot legal");
		status_bar.Set(0, Check_Position_Error(n));
		legal= false;
	}
}

void Position_Editor::Update_FEN()
{
	if (! legal)
	{
		fen_edit.Set_Text("");
		return;
	}
	fen_edit.Set_Text(FEN(position));
}

void Position_Editor::Update_EP()
{
	if (! legal)
	{
		ep_chooser.Set_Visible(false);
		return;
	}
	int ep_files= 0;
	Chess_Color to_move= position.Get_To_Move();
	for (int file= sq_a;  file <= sq_h;  ++file)
	{
		if (position.Get_Piece
		    (Make_Square
		     (file, 
		      to_move == cc_w ? sq_5 : sq_4)) 
		    == Piece(CC_Turn(to_move), pt_P)   &&
		    position.Get_Piece
		    (Make_Square
		     (file,
		      to_move == cc_w ? sq_6 : sq_3))
		    .Is_Empty()                                 &&
		    position.Get_Piece
		    (Make_Square
		     (file,
		      to_move == cc_w ? sq_7 : sq_2))
		    .Is_Empty()                                 &&
		    (position.Get_Piece
		     (Make_Invalid_Square
		      (file + 1,
		       to_move == cc_w ? sq_5 : sq_4))
		     == Piece(to_move, pt_P)        ||
		     position.Get_Piece
		     (Make_Invalid_Square
		      (file - 1,
		       to_move == cc_w ? sq_5 : sq_4))
		     == Piece(to_move, pt_P)))

			ep_files |= (1 << file);
	}
	if (! ep_files)
	{
		ep_chooser.Set_Visible(false);
		position.Get_EP_Square()= 0;
		return;
	}
	ep_chooser.Clear();
	int select_index= -1;

	char from_rank= to_move == cc_w ? '7' : '2';
	char to_rank=   to_move == cc_w ? '5' : '4';
	int ep_index_index= 0;
	for (int file= sq_a;  file <= sq_h;  ++file)
	{
		if (ep_files & (1 << file))
		{
			ep_index[ep_index_index++]= file;
			char f= file - sq_a + 'a';
			ep_chooser.Append
				(Format("%c%c-%c%c",
					f,
					from_rank,
					f,
					to_rank));
			if (position.Get_EP_Square() &&
			    file == Square_File(position.Get_EP_Square()))
			{
				select_index= ep_index_index-1;
			}
		}
	}

	Dbg_Int(select_index);
	ep_chooser.Append(Trl("Another move"));
	if (select_index == -1)
	{
		select_index= ep_chooser.Get_Count() - 1;
		position.Get_EP_Square()= 0;
	}
	ep_chooser.Set_Selected(select_index);
	ep_chooser.Set_Visible(true);
}

void
Position_Editor::Insert_New_Piece(int square)
{
	Dbg_Function(); 

	Dbg_Int(new_type.Get_Chooser().Get_Selected()); 

	if (new_type.Get_Chooser().Get_Selected() < 0)
		return;

	Piece new_piece((Chess_Color)(1 - new_type.Get_Chooser().Get_Selected() / 6),
			(Piece_Type)(new_type.Get_Chooser().Get_Selected() % 6));

	position.Insert_Piece(square, new_piece);

	Update_Legality();
	Update_EP();
	Update_FEN();
	Update_Castlings();
	Update_Legal();
	Modify();
}

bool
Position_Editor::Editor_Window_Try_Load(String fen, 
					Pointer <Position> position)
{
	return Read_FEN(fen, position());
}

void Position_Editor::Update_Legal()
{
	if (! legal_move_list.Is())
		return; 

	if (! legal_move_list.Get_Visible())
		return;

	if (! legal)
	{
		legal_move_list.Clear();
		return;
	}

	Generator g;
	g.New(position);
	legal_move_list.Set(g, persistent_notation.Get_Value());
}

void Position_Editor::Update_Castlings()
{
	Dbg_Function();

	const int squares[4][2]
		= {{sq_e1, sq_h1},
		   {sq_e1, sq_a1},
		   {sq_e8, sq_h8},
		   {sq_e8, sq_a8}};
	
	bool have_castlings= false;

	for (int i= 0;  i < 4;  ++i)
	{
		Dbg_Int(i);
		if (position.Get_Piece(squares[i][0]) == Piece(i / 2, pt_K) &&
		    position.Get_Piece(squares[i][1]) == Piece(i / 2, pt_R))
		{
			have_castlings= true;
			castling_check_boxes[i].Set_Visible(true);
			int moved_index= Castling_Moved(i);
			Dbg_Int(moved_index);
			castling_check_boxes[i].Set_Checked
				(position.Get_Moved(moved_index) == 0);
		}
		else
		{
			castling_check_boxes[i].Set_Visible(false);
		}
	}
	
	castling_widget.Set_Visible(have_castlings);
}

void Position_Editor::Update_PM()
{
	pm_edit.Set_Value(position.Get_Previous_Move_Count() / 2 + 1);
}

void Position_Editor::Wdg_Close()
{
	Retrieve_PM();
	persistent_notation.Close(); 
//	notation_editor.Close(); 
	Document_Editor <Position> ::Wdg_Close(); 
}

void Position_Editor::Retrieve_PM()
{
	position.Set_Previous_Move_Count
		((pm_edit.Get_Value() - 1) * 2 + position.Get_To_Move());
}

void Position_Editor::Remove_Piece(int square)
{
	Piece piece= position.Get_Piece(square);
	if (! piece.Is())
		return;
	position.Remove_Piece(square);
	board().Invalidate_Position();
	Update_Legality();
	Update_EP();
	Update_FEN();
	Update_Castlings();
	Update_Legal();
	Modify();
}

void Position_Editor::Normalize_Castlings()
{
	if (position.Get_Moved(1) != 0)
	{
		position.Get_Moved(0)= 1;
		position.Get_Moved(1)= 0;
		position.Get_Moved(2)= 1;
	}
	if (position.Get_Moved(4) != 0)
	{
		position.Get_Moved(3)= 1;
		position.Get_Moved(4)= 0;
		position.Get_Moved(5)= 1;
	}
}

void Position_Editor::Update_HM()
{
	hm_edit.Set_Value(position.Get_Halfmove_Clock()); 
}
