#ifndef POSITION_EDITOR_INCLUDED
#define POSITION_EDITOR_INCLUDED

#include "position_stream.hh"
#include "document_editor.hh"
#include "position.hh"
#include "radio_chooser.hh"
#include "file_dialog.hh"
#include "menued.hh"
#include "tool_bar.hh"
#include "status_bar.hh"
#include "text_button.hh"
#include "clean_board.hh"
#include "edit.hh"
#include "legal_move_list.hh"
#include "int_edit.hh"
#include "notation_editor.hh"
#include "persistent.hh"
#include "notation_stream.hh"

class Position_Editor
	:  public Document_Editor <Position> 
{
public:

	const Position &Editor_Window_Get_Value();
	static const char *Get_Help() 
		{
			return "Edit a chess position.\n";
		}
private:

	enum {msg_clear= Document_Editor <Position> ::msg_custom,
	      msg_color, msg_dragged,
	      msg_remove, msg_fen, msg_ep, msg_show_legal,
	      msg_show_fen, 
	      msg_castling, msg_castling_1, msg_castling_2,
	      msg_castling_3, 
	      msg_show_side_bar, msg_hm, msg_pm, msg_notation, 

	      msg_custom, 

	      menu_edit, menu_view,
	};


	Position position;
	/* MOVED[king squares] is always 0.
	   MOVED[rook squares] may be 0 even there is no rook on the
	   rook square.  In this case, the castling will be marked as
	   available when a rook is added. 
	 */

	bool legal;
	/* Whether position is legal.  Set by Update_Legality(). 
	 */
	int8_t ep_index[8];
	/* Set by Update_EP(), read by MR_Receive(msg_ep).
	   Index is of items in EP_CHOOSER.  Value is file of move. 
	 */

	Persistent <Notation, Notation_Editor> persistent_notation;

	Tool_Bar tool_bar;
	Bevel bevel;
	H_Packer middle_packer;
	V_Packer side_packer;
	Text_Button clear_button, reset_button;
	Radio_Chooser color_chooser;
	Radio_Chooser new_type;
	Radio_Chooser ep_chooser;
	Framed <V_Packer> castling_widget;
	Check_Box castling_check_boxes[4];
	H_Packer pm_packer;
	Text_Widget pm_text;
	Int_Edit pm_edit; 
	Legal_Move_List legal_move_list;
	H_Packer hm_packer;
	Text_Widget hm_text; 
	Int_Edit hm_edit;
	H_Packer fen_packer;
	Text_Widget fen_text;
	Edit fen_edit;
	Text_Button fen_button;
	Clean_Board board;
	Status_Bar status_bar;

	bool Document_Editor_Open(Pointer <Packer>);

	const char *EW_Type_Name() const 
		{
			return Trl("Chess Position"); 
		}
	const char *EW_Short_Type_Name() const
		{
			return Trl("Position"); 
		}

	int Editor_Window_Use();
	void Editor_Window_Set_Value(Pointer <const Position>);
	bool Editor_Window_Change_Value(const Position &);
	void DE_New();
	bool Editor_Window_Try_Load(String, Pointer <Position>);

	void MR_Receive(int);
	void Menued_Get(int menu, Menued::Entries &entries);

	void Update_Board();
	/* When POSITION has changed.  Update the board and
	   COLOR_CHOOSER.  Calls all other Update...() functions. 
	 */
	void Update_Legality();
	/* Set LEGAL and update status line.
	 */
	void Update_EP();
	/* Update ep chooser.  Called after Update_Legality() because
	   is reads LEGAL. 
	 */
	void Update_FEN();
	/* Update FEN edit text.  Must be called after Update_EP(). 
	 */
	void Update_Castlings();
	/* Update castling check boxes. 
	 */
	void Update_PM();
	void Update_Legal();
	/* Update legal move list if visible. 
	 */
	void Insert_New_Piece(int square);
	void Wdg_Close();
	void Retrieve_PM();
	void Remove_Piece(int square);
	void Normalize_Castlings();
	void Update_HM();

	static int Castling_Moved(int index)
		{
			return index / 2 * 3 + 2 - index % 2 * 2;
		}
};

#endif /* ! POSITION_EDITOR_INCLUDED */

