#ifndef SQUARE_INCLUDED
#define SQUARE_INCLUDED

/* A square on a chess board 
 */

#include "string.hh"
#include "chess_color.hh"
#include "inline.hh"

//typedef int Square;

/*  
          A    B    C    D    E    F    G    H
      ||---------------------------------------||
    8 || 91 | 92 | 93 | 94 | 95 | 96 | 97 | 98 || 8
      ||---------------------------------------||
    7 || 81 | 82 | 83 | 84 | 85 | 86 | 87 | 88 || 7
      ||---------------------------------------||
    6 || 71 | 72 | 73 | 74 | 75 | 76 | 77 | 78 || 6
      ||---------------------------------------||
    5 || 61 | 62 | 63 | 64 | 65 | 66 | 67 | 68 || 5
      ||---------------------------------------||
    4 || 51 | 52 | 53 | 54 | 55 | 56 | 57 | 58 || 4
      ||---------------------------------------||
    3 || 41 | 42 | 43 | 44 | 45 | 46 | 47 | 48 || 3
      ||---------------------------------------||
    2 || 31 | 32 | 33 | 34 | 35 | 36 | 37 | 38 || 2
      ||---------------------------------------||
    1 || 21 | 22 | 23 | 24 | 25 | 26 | 27 | 28 || 1
      ||---------------------------------------||
          A    B    C    D    E    F    G    H
*/

//typedef int Coord;
/* The file or rank number, zero based.  File A is 0, H is 7. */

/* FILE and RANKs:  always zero-based:  a=0, h=7, "1"=0, "8"= 7. 
 */

//static const
//int
enum {sq_a1= 21, sq_b1= 22, sq_c1= 23, sq_d1= 24, sq_e1= 25, 
      sq_f1= 26, sq_g1= 27, sq_h1= 28, sq_a8= 91, sq_b8= 92, 
      sq_c8= 93, sq_d8= 94, sq_e8= 95, sq_f8= 96, sq_g8= 97, 
      sq_h8= 98, sq_a= 0, sq_b= 1, sq_c= 2, sq_d= 3, sq_e= 4, sq_f= 5,
      sq_g= 6, sq_h= 7, sq_1= 0, sq_2= 1, sq_3= 2, sq_4= 3, sq_5= 4,
      sq_6= 5, sq_7= 6, sq_8= 7};

Inline
void Square_Turn(int &square)
	/* The same square when the board is turned. */
{
	square= 119 - square;
}

String Format_Square(int square);

Inline
int
Square_File(int square) 
	/* the file of the square */
{
	int file= square % 10 - 1; 
	Assert (file >= 0 && file < 8);
	return file;
}

Inline
int Square_Rank(int square) 
	/* the rank of the square */
{
	int result= square / 10 - 2;
	Assert (result >= 0 && result < 8);
	return result;
}

Inline
bool
Is_Inside_Square(int square)
	/* Whether the square is one of 64 chess board squares. 
	 */
{
	return square >= 21 && square < 99 && square % 10 > 0 && square % 10 < 9; 
}

Inline
int
Make_Invalid_Square(int file, int rank)
     /* May be invalid. 
      */
{
  return 10 * rank + file + 21;
}

Inline
int
Make_Square(int file, 
	    int rank)
{
	Assert (file >= 0 && file < 8 && rank >= 0 && rank < 8);

//	return 10 * rank + file + 21;
	return Make_Invalid_Square(file, rank);
}

Inline
Chess_Color
Square_Color(int s)
	/* S must be on the chess board */
{
	Assert (Is_Inside_Square(s));
	return ((s / 10 + s) % 2 == 0) ? cc_w : cc_b;
}

#if 0

Inline
bool
Check_Square(int square)
{
	return (Square_File(square) >= 0
		&& Square_File(square) < 8
		&& Square_Rank(square) >= 0
		&& Square_Rank(square) < 8);
}
#endif /* 0 */ 

Inline
int
Square_Mirror(int square)
{
	return Make_Square(Square_File(square),
			   7 - Square_Rank(square));
}

#endif /* ! SQUARE_INCLUDED */
