#ifndef TIME_INCLUDED
#define TIME_INCLUDED

/* See `doc/time.hh' for documentation. 
 */

#include "time.config.hh"

#include "inline.hh"

#ifndef TIME_NEW
/* Alternative `clock_t'-based implementation. 
 */

#include <time.h>

typedef clock_t Time;

Inline
Time
Get_Time() 
{
	clock_t result= clock();

	return result; 
}

Inline
Time
ms_To_Time(int ms) 
{
	return ms * CLOCKS_PER_SEC / 1000; 
}

Inline
int
Time_To_ms(Time time)
{
#if CLOCKS_PER_SEC >= 10000
	return time / (CLOCKS_PER_SEC / 1000);
#else
	return time * 1000 / CLOCKS_PER_SEC;
#endif
}

#else /* TIME_NEW */

/* New "struct timeval"-based implementation. 
 */

#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>

typedef struct timeval Time;

Inline
Time
Get_Time()
{
	struct rusage usage;

	getrusage(RUSAGE_SELF, & usage);

	struct timeval result;
	result.tv_sec= (usage.ru_utime.tv_sec + usage.ru_stime.tv_sec
		    + (usage.ru_utime.tv_usec + usage.ru_stime.tv_usec) / 1000000); 
	result.tv_usec= (usage.ru_utime.tv_usec + usage.ru_stime.tv_usec) % 1000000; 

	return result;
}

Inline
Time 
ms_To_Time(int ms)
{
	struct timeval result;
    
	result.tv_sec= ms / 1000;
	result.tv_usec= ms % 1000 * 1000;

	return result;
}

Inline
int
Time_To_s(Time time)
{
	return time.tv_sec; 
}

Inline
int
Time_To_ms(Time time)
{
	return time.tv_sec * 1000 + time.tv_usec / 1000;
}

Inline
int
Time_To_us(Time time)
{
	return time.tv_sec * 1000000 + time.tv_usec;
}

Inline
Time operator - (Time b, Time a)
{
	struct timeval rv;

	rv.tv_sec= b.tv_sec - a.tv_sec;
	rv.tv_usec= b.tv_usec - a.tv_usec;
    
	if (rv.tv_usec + 1000000 < 1000000)
	{
		rv.tv_usec += 1000000;
		-- rv.tv_sec;
	}

	return rv;
}

#endif /* TIME_NEW */

#endif /* ! TIME_INCLUDED */
