#ifndef WIDGET_INCLUDED
#define WIDGET_INCLUDED

#include "window.hh"
#include "pointer.hh"
#include "this.hh"
#include "vector.hh"
#include "message_receiver.hh"
#include "stretchness.hh"
#include "canvas.hh"
#include "point.hh"
#include "rect.hh"
#include "mouse_button.hh"
#include "mouse_action.hh"
#include "key.hh"
#include "menued.hh"

enum Mouse_Cursor {ms_arrow= 0, ms_move, ms_horizontal, ms_vertical,
		   ms_swne, ms_nwse, ms_no, ms_hand, ms_layer,
		   ms_count};


class Widget
	:  public Gtk_Object, 
	   virtual public Message_Receiver
{
public:

	enum {msg_ignore= -1,

	      msg_ok= 0, msg_apply, msg_reset,
	      msg_cancel,
	      
	      msg_custom,
	      
	      unit= 20}; 

	Widget();

	virtual ~Widget();

	bool Open(Pointer <Widget> parent); 

	bool Open(Pointer <Widget> new_parent,
		  GtkWidget *new_gtkwidget,
		  bool open_window= true); 
	/* NEW_GTKWIDGET as returned by a GTK function.
	   NEW_GTKWIDGET may be NULL. 
	*/

	bool Append(Pointer <Widget> child); 

//	void MR_Send(int msg);

	void MR_Receive(int msg);

	int Get_Children_Count() const
		{
			return *children; 
		}

	void Invalidate()
		{
			Dbg_Function_();

			gtk_widget_queue_draw(Get());
		}
#if 0
	void Invalidate_Clear()
		{
			gtk_widget_queue_resize(Get());
		}
#endif /* 0 */ 
	void Invalidate(const Rect &rect)
		{
			Dbg_Function_();
			Dbg_("\trect= (%4d, %4d)  %4dx%4d",
			     rect.origin.x, rect.origin.y,
			     rect.size.x, rect.size.y);

			gtk_widget_queue_draw_area
				(Get(),
				 rect.origin.x,
				 rect.origin.y,
				 rect.size.x,
				 rect.size.y); 
//			Invalidate(& r);
		}
	Window &Get_Window();

	const Window &Get_Window() const 
		{
			if (gtk_window.Is())
				return gtk_window;
			else
				return parent().Get_Window(); 
		}

	void Activate_Window()
		{
			GdkWindow *w= gtk_widget_get_parent_window(Get());

			if (w == NULL)
				return;
			gdk_window_raise(w);
			/* TODO:  give input focus to window */ 
//	gtk_widget_grab_focus(Get());
//			Get_Window().Activate();
		}

	void Set_Transient(const Widget &widget);

	virtual GtkWindow *Get_Gtk_Window() const
		{
			return GTK_WINDOW(Get_Window().Get());
		}
	Widget &Get_Child(int n) const
		{
			return children[n](); 
		}

	bool Capture(bool /* not_children */ = false)
		{
			return false; 
		}
	void Release_Capture()
		{
		}
	void Invalidate_Size()
		{
			Dbg_Function();
			gtk_widget_queue_resize(Get());
		}
	void Set_Visible(bool visible);

	bool Get_Visible() const
		{
			return GTK_WIDGET_VISIBLE(Get()); 
		}
	void Set_Bounds(Rect) const
		{
		}

	void Wdg_Close(); 

	virtual bool Wdg_Key_Press(const Key &)
		{
			return false; 
		}

	virtual void Wdg_Activate(bool /* active */ )
		{
		}
	void Activate()
		{
		}
	void Parent_Mouse(Mouse_Action a, 
			  Mouse_Button b, 
			  Point p,
			  bool ac)
		{
			Widget::Wdg_Mouse(a, b, p, ac); 
		}
	virtual void Wdg_Mouse(Mouse_Action, Mouse_Button, Point,
			       bool);

	Pointer <Widget> Get_Parent()
		{
			return parent; 
		}

	void Parent_Send_Message(int id)
		{
			if (Get_Parent().Is())
				Get_Parent()().MR_Send(id);
		}
#if 0
	void Set_Size(Point)
		{
		}
#endif /* 0 */ 
	void Parent_Receive_Message(int id)
		{
			Assert (id >= -1);

			if (id != -1)
				Get_Parent()().MR_Receive(id);
		}

	void Set_Mouse_Cursor(Mouse_Cursor mc);

	virtual void Wdg_Append_Child(Widget &);
		/* Widgets that can have children (as V_Packer) implement this to
		   call e.g. gtk_box_pack_start(). 
		*/
	void Set_Index(int index)
		{
			gtk_box_reorder_child(GTK_BOX(Get_Parent()().Get()),
					      Get(),
					      index);
		}
//	void Set_Selection(String);
#if 0
	String Get_Selection()
		{
			return "";
		}
#endif /* 0 */ 

	static void Loop()
		{
			Window::Loop();
		}
	static int Get_Depth()
		{
			Assert (depth > 0);
			return depth; 
		}
	static void Set_Transient(GtkWindow *window,
				  GtkWindow *for_window)
		{
			gtk_window_set_transient_for(window,
						     for_window); 
		}

private:

	Window gtk_window;   
	Vector <Pointer <Widget> > children;
	Pointer <Widget> parent;

	static int depth;

	void Wdg_Destroy(); 

	virtual void Wdg_Draw(Canvas &)
		{
		}

	void Wdg_Do_Close(); 
	virtual String Wdg_Init_Caption() const
		{
			return "Window"; 
		}
	virtual Point Wdg_Get_Window_Size() const
		/* Return x=-1 for no resizing. 
		 */
		{
			return Point(600, 400); 
		}

//	static gint Idle(gpointer);
};

#endif /* ! WIDGET_INCLUDED */
