#include "xboard_engine_session.hh"

#include "parser.hh"
//#include "san.hh"
#include "io.hh"
//#include "scan.hh"
#include "simple_poll.hh"
#include "read_line.hh"
#include "write_formatted.hh"
#include "notation.hh"

void
XBoard_Engine_Session::Process_Input(String line)
{
	Parser parser(line);
	parser.Skip_Space();
	String word= parser.Skip_Word();
	if (word == "quit")
	{
		quit= true; 
	}
	else if (word == "go")
	{
		Stop();
		Evaluator::Run(); 
	}
	else if (word == "new")
	{
		Stop();
		game.Open();
	}
	else if (word == "?")
	{
		if (Evaluator::Is())
		{
			Assert (*game.Get_Generator() > 0);
			int best_move= Evaluator::Get_Best_Move();
			Stop();
			String move_text= san_notation.Format_Move_Name
				(game.Get_Generator().Get_Move_Name(best_move)); 
//			game.Play(best_move);
			game.Append_Move(best_move);
			Write_Formatted(IO::out,
				    "move %s\n",
					move_text());
		}
	}
	else if (word == "time")
	{
		parser.Skip_Space();
		String time= parser.Skip_Word();
		int time_cs;
		if (word.Is() && 1 == sscanf(time(), "%d", & time_cs))
		{
			int time_s= time_cs / (25 * 100) >? 1; 
			Dbg("time= %d s", time_s); 
			Evaluator::Set_Time(time_s);
		}
		else
		{
			Write_Formatted(IO::out,
				    "Error (wrong parameter): wrong parameter\n");
		}
	}
	else if (word == "post")
	{
		post= true; 
	}
	else if (word == "nopost")
	{
		post= false;
	}
	else if (word == "remove")
	{
		if (game.Get_Overall_Location() >= 2)
		{
			Stop();
			game.Go_To_Overall_Location(game.Get_Overall_Location() - 2);
		}
	}
	else /* move name */ 
	{
		int move_index= game.Get_Generator().Get_e2e4_Notation_Move(word); 
		if (move_index != -1)
		{
			Dbg("\tScanned as #%d.", move_index);
			Stop();
			game.Append_Move(move_index);
			if (*game.Get_Generator() == 0)
			{
				if (game.Get_Position().Is_Check())
				{
					if (game.Get_Position().Get_To_Move() == cc_w)
						Write_Formatted(IO::out,
							    "0-1 {Black mates}\n");
					else
						Write_Formatted(IO::out,
							    "1-0 {White mates}\n");
				}
				else
				{
					Write_Formatted(IO::out,
						    "1/2-1/2 {Stalemate}\n");
				}
			}
			else if (*game.Get_Generator() == 1)
			{
				String move_text= san_notation.Format_Move_Name
					(game.Get_Generator().Get_Move_Name(0)); 
				Write_Formatted(IO::out,
					    "move %s\n",
					    move_text());
				game.Append_Move(0);
			}
			else
				if (! Evaluator::Run())
					Dbg("\tEvaluator not started.");
				else
					Dbg("\tEvaluator started."); 
		}
		else
		{
			Write_Formatted(IO::out,
				    "Error (unknown command or no such move): %s\n",
				    word());
		}
	}
}

void
XBoard_Engine_Session::Run()
{
	IO::in.Set_Block(false); 
	Input_Reader::Open(IO::in); 

	game.Open();

	Evaluator::Open(game.Get_Position()); 

	Evaluator::Set_Settings
		(Evaluator_Settings
		 (Evaluator_Settings::ti_infinite,
		  10,
		  false,
		  true)); 

	quit= false;
	post= false;
			
	while (! quit)
	{
		Simple_Poll(); 
	}

	if (Evaluator::Is())
		Evaluator::Terminate(true);
	Evaluator::Close(); 
}

void
XBoard_Engine_Session::IR_Input()
{
	String line;
	while (Read_Line(IO::in, line))
	{
		if (line.Is())
		{
			Process_Input(line);
		}
	}

}


void XBoard_Engine_Session::EC_Terminate
(bool really, 
 bool  engine_finished  ) 
{
	Dbg("EC_Terminate(%s)", really ? "really" : "! really");

	if (really)
		return;

	if (! engine_finished)
		return;

	int best_move= Evaluator::Get_Best_Move();

	if (best_move == -1)
	  {
	    return;
	  }

	String text= game.Get_Generator().Get_Simple_Move_Name(best_move);
	Write_Formatted(IO::out,
		    "move %s\n",
		    text());
	game.Append_Move(best_move);
}


void
XBoard_Engine_Session::EC_Main(int)
{
	Dbg("EC_Main()");

	if (post)
	{
		String move= san_notation.Format_Move_Name
			(game.Get_Generator().Get_Move_Name(Evaluator::Get_Best_Move()));
		Write_Formatted(IO::out,
			    "%d %d %d %d %s\n",
			    Evaluator::Get_Depth(),
			    Evaluator::Get_Score(),
			    Evaluator::Get_Date_ms() / 10,
			    Evaluator::Get_Knots(),
			    move());
	}
}
